SoXmlWrapElement subclass: SoapWrapElement [
    
    <comment: 'Convenient XML Element which is contained in SoapElement

---
mu 1/4/2002 22:54'>
    <category: 'SOAP-Core-Base'>

    SoapWrapElement class >> arrayTypeMatchString [
	<category: 'constants'>
	^'*arrayType'
    ]

    arrayItemType [
	<category: 'actions'>
	| arrType |
	arrType := self arrayType.
	^arrType isNil 
	    ifTrue: [SoapConstants xsdAnyType]
	    ifFalse: 
		[| end |
		end := arrType findString: '[' startingAt: 0.
		arrType copyFrom: 1 to: end - 1]
    ]

    arrayType [
	<category: 'actions'>
	^self getAttributeLike: self class arrayTypeMatchString
    ]

    attributesHolder [
	<category: 'accessing'>
	attributesHolder isNil 
	    ifTrue: [attributesHolder := SoapAttributesHolder new].
	^attributesHolder
    ]

    declareEncodingIn: nameSpacePrefix uri: uriString [
	<category: 'actions'>
	self 
	    putAttributeNamed: nameSpacePrefix , ':' , SoapConstants encodingStyle
	    value: uriString
    ]

    declareSoapEncNamespace [
	<category: 'actions'>
	self declareNamespace: SoapConstants soapEncPrefix
	    uri: SoapConstants soapEncURI
    ]

    declareTypicalEncoding [
	<category: 'actions'>
	self declareEncodingIn: SoapConstants soapEnvPrefix
	    uri: SoapConstants soapEncURI
    ]

    declareTypicalNamespaces [
	<category: 'actions'>
	self declareSoapEncNamespace.
	self declareXsdNamespace.
	self declareXsiNamespace
    ]

    declareXsdNamespace [
	<category: 'actions'>
	self declareNamespace: SoapConstants xsdPrefix uri: SoapConstants xsdURI
    ]

    declareXsiNamespace [
	<category: 'actions'>
	self declareNamespace: SoapConstants xsiPrefix uri: SoapConstants xsiURI
    ]

    encodingStyleURIs [
	"ToDo: take a more serious approach...Namespace conscious"

	<category: 'actions'>
	| encs |
	encs := self attributesHolder encodingStyle.
	((encs isNil or: [encs isEmpty]) and: [self parent notNil]) 
	    ifTrue: [^self parent encodingStyleURIs].
	^(SoPortableUtil substringsFrom: encs) asArray
    ]

    href [
	<category: 'actions'>
	^self getAttributeNamed: SoapConstants href
    ]

    href: aString [
	<category: 'actions'>
	^self putAttributeNamed: SoapConstants href value: aString
    ]

    id [
	<category: 'actions'>
	^self getAttributeNamed: SoapConstants id
    ]

    id: aString [
	<category: 'actions'>
	^self putAttributeNamed: SoapConstants id value: aString
    ]

    isArrayType [
	<category: 'testing'>
	^self hasAttributeLike: self class arrayTypeMatchString
    ]

    size [
	<category: 'actions'>
	| arrType |
	arrType := self arrayType.
	^arrType isNil 
	    ifTrue: [self children size]
	    ifFalse: 
		[| start |
		start := arrType findString: '[' startingAt: 0.
		(arrType copyFrom: start + 1 to: arrType size - 1) asNumber]
    ]

    xsiType [
	<category: 'actions'>
	^self getAttributeNamed: SoapConstants xsiType
    ]

    xsiType: aString [
	<category: 'actions'>
	^self putAttributeNamed: SoapConstants xsiType value: aString
    ]
]



SoapWrapElement subclass: SoapEncArray [
    | encoder |
    
    <comment: nil>
    <category: 'SOAP-Core-Encoding'>

    SoapEncArray class >> encoder: aSoapEncoder [
	<category: 'instance creation'>
	^(self new)
	    encoder: aSoapEncoder;
	    yourself
    ]

    SoapEncArray class >> fromArray: anArray [
	<category: 'instance creation'>
	^self fromArray: anArray named: 'Array'
    ]

    SoapEncArray class >> fromArray: anArray named: elemName [
	<category: 'instance creation'>
	^self 
	    fromArray: anArray
	    named: elemName
	    elementType: SoapConstants xsdAnyType
    ]

    SoapEncArray class >> fromArray: anArray named: elemName elementType: anXSDType [
	<category: 'instance creation'>
	^self 
	    fromArray: anArray
	    named: elemName
	    elementType: anXSDType
	    containerType: nil
	    namespace: nil
    ]

    SoapEncArray class >> fromArray: anArray named: elemName elementType: elementXSDType containerType: arrayXSDType namespace: namespaceURI [
	<category: 'instance creation'>
	| inst |
	inst := self new.
	^inst 
	    fromArray: anArray
	    named: elemName
	    elementType: elementXSDType
	    containerType: arrayXSDType
	    namespace: namespaceURI
    ]

    encoder [
	"Answer the value of encoder"

	<category: 'accessing'>
	^encoder
    ]

    encoder: anObject [
	<category: 'accessing'>
	encoder := anObject
    ]

    fromArray: anArray [
	<category: 'initialize-release'>
	^self fromArray: anArray named: 'Array'
    ]

    fromArray: anArray named: elemName [
	<category: 'initialize-release'>
	^self 
	    fromArray: anArray
	    named: elemName
	    elementType: SoapConstants xsdAnyType
    ]

    fromArray: anArray named: elemName elementType: anXSDType [
	<category: 'initialize-release'>
	^self 
	    fromArray: anArray
	    named: elemName
	    elementType: anXSDType
	    containerType: nil
	    namespace: nil
    ]

    fromArray: anArray named: elemName elementType: elementXSDType containerType: arrayXSDType namespace: namespaceURI [
	<category: 'initialize-release'>
	| encPrefix |
	self name: elemName.
	encPrefix := SoapConstants soapEncPrefix.
	self declareNamespace: encPrefix uri: SoapConstants soapEncURI.
	self declareXsdNamespace.
	self declareXsiNamespace.
	namespaceURI notNil 
	    ifTrue: 
		[| prefix |
		prefix := arrayXSDType isNil 
			    ifTrue: [SoXmlUtil randomPrefix]
			    ifFalse: [SoXmlUtil prefixFrom: arrayXSDType].
		self declareNamespace: prefix uri: namespaceURI].
	self putAttributeNamed: SoapConstants arrayType
	    value: elementXSDType , '[' , anArray size printString , ']'.
	anArray do: 
		[:each | 
		(each isNil or: [each == #nil]) 
		    ifFalse: 
			[| item child |
			item := SoapVariable name: 'item' value: each.
			child := (self encoder ifNil: [item encoder]) encodeSoapVariable: item.
			self addChild: child]]
    ]

    isArrayType [
	"for performance"

	<category: 'testing'>
	^true
    ]

    trimElementTypeAttributePrefix [
	<category: 'private'>
	self attributesHolder trimAttributePrefixNamed: SoapConstants arrayType
    ]

    trimForNested [
	<category: 'actions'>
	self beShortName.
	self undeclareNamespaces.
	self trimElementTypeAttributePrefix
    ]
]



SoXmlAttributesHolder subclass: SoapAttributesHolder [
    | encodingStyle |
    
    <comment: 'A Container for XML attributes.


---
mu 5/21/2001 22:24'>
    <category: 'SOAP-Core-Base'>

    encodingStyle [
	<category: 'accessing'>
	^encodingStyle
    ]

    encodingStyle: aString [
	<category: 'accessing'>
	encodingStyle := aString
    ]

    putAttributeNamed: attribName value: attribValue [
	<category: 'actions'>
	"ToDo: take a more serious approach...Namespace conscious"

	| encs |
	encs := SoapConstants encodingStyle.
	('*:' , encs match: attribName) ifTrue: [self encodingStyle: attribValue].
	super putAttributeNamed: attribName value: attribValue
    ]
]



SoXmlConstants subclass: SoapConstants [
    
    <comment: 'SOAP Constants provider.

(I hate to use Pool)
---
mu 5/3/2001 19:32'>
    <category: 'SOAP-Core-Base'>

    SoapConstants class >> actor [
	<category: 'attributes'>
	^self at: #actor ifAbsentPutValue: 'actor'
    ]

    SoapConstants class >> arrayType [
	<category: 'attributes'>
	^self at: #arrayType
	    ifAbsentPutValue: self soapEncPrefixColon , 'arrayType'
    ]

    SoapConstants class >> client [
	<category: 'fault codes'>
	^self at: #client ifAbsentPutValue: 'Client'
    ]

    SoapConstants class >> encodingStyle [
	<category: 'attributes'>
	^self at: #encodingStyle ifAbsentPutValue: 'encodingStyle'
    ]

    SoapConstants class >> href [
	<category: 'attributes'>
	^self at: #href ifAbsentPutValue: 'href'
    ]

    SoapConstants class >> id [
	<category: 'attributes'>
	^self at: #id ifAbsentPutValue: 'id'
    ]

    SoapConstants class >> mustUnderstand [
	<category: 'fault codes'>
	^self at: #mustUnderstand ifAbsentPutValue: 'MustUnderstand'
    ]

    SoapConstants class >> nullEncPrefix [
	<category: 'prefixes'>
	^self at: #nullEncPrefix ifAbsentPutValue: 'Null-ENC'
    ]

    SoapConstants class >> nullEncPrefixColon [
	<category: 'prefixes'>
	^self withColon: self nullEncPrefix
    ]

    SoapConstants class >> nullEncURI [
	"non standard squeak specific encoding - MU"

	<category: 'URIs'>
	^self at: #soapNullEncURI
	    ifAbsentPutValue: 'http://www.mars.dti.ne.jp/~umejava/smalltalk/soapOpera/null/encoding/'
    ]

    SoapConstants class >> returnValueAccessor [
	<category: 'rpc'>
	^self at: #returnValueAccessor ifAbsentPutValue: 'result'
    ]

    SoapConstants class >> server [
	<category: 'fault codes'>
	^self at: #server ifAbsentPutValue: 'Server'
    ]

    SoapConstants class >> soapActorNextURI [
	<category: 'URIs'>
	^self at: #soapActorNextURI
	    ifAbsentPutValue: 'http://schemas.xmlsoap.org/soap/actor/next'
    ]

    SoapConstants class >> soapEncPrefix [
	<category: 'prefixes'>
	^self at: #soapEncPrefix ifAbsentPutValue: 'enc'
    ]

    SoapConstants class >> soapEncPrefixColon [
	<category: 'prefixes'>
	^self withColon: self soapEncPrefix
    ]

    SoapConstants class >> soapEncURI [
	"ToDo: swith 1.2 value: http://www.w3.org/2001/12/soap-encoding"

	<category: 'URIs'>
	^self at: #soapEncURI
	    ifAbsentPutValue: 'http://schemas.xmlsoap.org/soap/encoding/'
    ]

    SoapConstants class >> soapEnvPrefix [
	<category: 'prefixes'>
	^self at: #soapEnvPrefix ifAbsentPutValue: 'env'
    ]

    SoapConstants class >> soapEnvPrefixColon [
	<category: 'prefixes'>
	^self withColon: self soapEnvPrefix
    ]

    SoapConstants class >> soapEnvURI [
	<category: 'URIs'>
	^self at: #soapEnvURI
	    ifAbsentPutValue: 'http://schemas.xmlsoap.org/soap/envelope/'
    ]

    SoapConstants class >> soapSixxEncURI [
	"sixx encoding - MU"

	<category: 'URIs'>
	^self at: #soapSixxEncURI
	    ifAbsentPutValue: 'http://www.mars.dti.ne.jp/~umejava/smalltalk/sixx/encoding/'
    ]

    SoapConstants class >> soapSrpEncURI [
	"camp smalltalk SRP encoding - MU"

	<category: 'URIs'>
	^self at: #srpEncURI
	    ifAbsentPutValue: 'http://wiki.cs.uiuc.edu/CampSmalltalk/SRP/encoding/'
    ]

    SoapConstants class >> sqEncPrefix [
	<category: 'prefixes'>
	^self at: #sqEncPrefix ifAbsentPutValue: 'Squeak-ENC'
    ]

    SoapConstants class >> sqEncPrefixColon [
	<category: 'prefixes'>
	^self withColon: self sqEncPrefix
    ]

    SoapConstants class >> versionMismatch [
	<category: 'fault codes'>
	^self at: #versionMismatch ifAbsentPutValue: 'VersionMismatch'
    ]

    SoapConstants class >> xsiNil [
	"shortcut for performance"

	"xsi:null is Soap1.1"

	"xsi:nil is Soap1.2"

	<category: 'attributes'>
	^self at: #xsiNil ifAbsentPutValue: self xsiPrefixColon , 'nil'
    ]
]



String extend [

    soapIdKey [
	<category: '*soap-core'>
	^self hash
    ]

]



Object class extend [

    fromXmlElement: aSoapXmlElement [
	<category: '*soap-core'>
	^self fromXmlElement: aSoapXmlElement with: nil
    ]

    fromXmlElement: aSoapXmlElement with: aSoapDecoder [
	"Conrtol all deserialization from SoapWrapElement"

	"This is only useful for customized Soap encoding"

	<category: '*soap-core'>
	^self new
    ]

]



Object extend [

    asXmlElement: aSoapVariable [
	<category: '*soap-core'>
	^self asXmlElement: aSoapVariable with: nil
    ]

    asXmlElement: aSoapVariable with: aSoapEncoder [
	"Return a SoapWrapElement from aSoapVariable"

	"This is only useful for customized Soap encoding"

	<category: '*soap-core'>
	^self
    ]

    soapIdKey [
	<category: '*soap-core'>
	^self identityHash
    ]

    soapIgnorableInstVarNames [
	"Override if you wish"

	"If you would like to omit some inst vars in serialization, List their names here."

	<category: '*soap-core'>
	^#()
    ]

]



Object subclass: SoapAction [
    | intent |
    
    <category: 'SOAP-Core-Http'>
    <comment: '''SOAPAction: ''. in http header.

MU 10/5/2002 01:29'>

    SoapAction class >> intent: aString [
	<category: 'instance creation'>
	^self new intent: aString
    ]

    SoapAction class >> intentByRequestURI [
	<category: 'instance creation'>
	^self intent: ''
    ]

    SoapAction class >> noIntent [
	<category: 'instance creation'>
	^self intent: nil
    ]

    headerString [
	<category: 'constants'>
	^'SOAPAction'
    ]

    intent [
	<category: 'accessing'>
	^intent
    ]

    intent: aValue [
	<category: 'accessing'>
	intent := aValue
    ]

    intentString [
	<category: 'actions'>
	| headerStream |
	self intent ifNil: [^''].
	headerStream := WriteStream on: ''.
	headerStream nextPut: $".
	headerStream nextPutAll: self intent.
	headerStream nextPut: $".
	^headerStream contents
    ]

    printOn: aStream [
	<category: 'printing'>
	aStream nextPutAll: self headerString , ': '.
	aStream nextPutAll: self intentString.
	aStream cr.
	aStream nextPut: Character lf
    ]
]



Object subclass: SoapCallEntry [
    | locator |
    
    <category: 'SOAP-Core-RPC'>
    <comment: 'I represent a SOAP RPC access point.

---
MU 5/12/2002 00:47'>

    SoapCallEntry class [
	| defaultSendingPort |
	
    ]

    SoapCallEntry class >> defaultSendingPort [
	<category: 'accessing'>
	^defaultSendingPort
    ]

    SoapCallEntry class >> defaultSendingPort: portNumber [
	<category: 'accessing'>
	defaultSendingPort := portNumber
    ]

    SoapCallEntry class >> initialize [
	"SoapCallEntry initialize"

	<category: 'class initialization'>
	defaultSendingPort := 8823
    ]

    SoapCallEntry class >> local [
	<category: 'instance creation'>
	^self tcpHost: 'localhost' port: self defaultSendingPort
    ]

    SoapCallEntry class >> local: portNumberInteger [
	<category: 'instance creation'>
	^self new 
	    locator: (SoapTcpLocator host: 'localhost' port: portNumberInteger)
    ]

    SoapCallEntry class >> tcpHost: hostnameString [
	<category: 'instance creation'>
	^self tcpHost: hostnameString port: self defaultSendingPort
    ]

    SoapCallEntry class >> tcpHost: hostnameString port: portNumberInteger [
	"In many cases, these values are used for setting router server URI"

	<category: 'instance creation'>
	^self new 
	    locator: (SoapTcpLocator host: hostnameString port: portNumberInteger)
    ]

    locator [
	<category: 'accessing'>
	^locator
    ]

    locator: aValue [
	<category: 'accessing'>
	locator := aValue
    ]

    newCall [
	<category: 'actions'>
	^SoapCall locator: self locator
    ]
]



Object subclass: SoapConfigurationMap [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>

    SoapConfigurationMap class [
	| implDict |
	
    ]

    SoapConfigurationMap class >> at: aSymbol [
	<category: 'actions'>
	^self implDict at: aSymbol ifAbsent: []
    ]

    SoapConfigurationMap class >> at: aSymbol put: realClass [
	<category: 'actions'>
	^self implDict at: aSymbol put: realClass
    ]

    SoapConfigurationMap class >> implDict [
	<category: 'accessing'>
	implDict isNil ifTrue: [implDict := IdentityDictionary new].
	^implDict
    ]

    SoapConfigurationMap class >> initImplDict [
	<category: 'class initialization'>
	implDict := nil
    ]

    SoapConfigurationMap class >> initialize [
	"SoapConfigurationMap initialize"

	<category: 'class initialization'>
	self initImplDict.
	Smalltalk at: self rootImplClassName
	    ifPresent: [:found | found initializeAll]
    ]

    SoapConfigurationMap class >> initializeAll [
	"SoapConfigurationMap initializeAll"

	<category: 'class initialization'>
	self allSubclassesDo: [:each | each initialize]
    ]

    SoapConfigurationMap class >> isKom6plus [
	<category: 'utility'>
	^Smalltalk includesKey: #HttpService
    ]

    SoapConfigurationMap class >> isNG [
	<category: 'utility'>
	^Smalltalk includesKey: #ComancheHttpService
    ]

    SoapConfigurationMap class >> isServerInstalled [
	<category: 'utility'>
	^Smalltalk includesKey: #SoapServerSetup
    ]

    SoapConfigurationMap class >> removeAt: aSymbol [
	<category: 'actions'>
	^self implDict removeKey: aSymbol ifAbsent: []
    ]

    SoapConfigurationMap class >> rootImplClassName [
	"subclass should return a class name"

	<category: 'factory'>
	^nil
    ]
]



Object subclass: SoapContext [
    | propDict |
    
    <category: 'SOAP-Core-RPC'>
    <comment: 'I represent a SOAP RPC context.

---
MU 5/12/2002 00:47'>

    at: aSymbol [
	<category: 'actions-basic'>
	^self propDict at: aSymbol
    ]

    at: aSymbol ifAbsent: aBlock [
	<category: 'actions-basic'>
	^self propDict at: aSymbol ifAbsent: aBlock
    ]

    at: aSymbol put: anObject [
	<category: 'actions-basic'>
	^self propDict at: aSymbol put: anObject
    ]

    encodingStyleURIs [
	<category: 'actions'>
	^self at: #encodingStyleURIs ifAbsent: [#()]
    ]

    encodingStyleURIs: anArrayOfEncodingStyleURI [
	<category: 'actions'>
	self at: #encodingStyleURIs put: anArrayOfEncodingStyleURI
    ]

    printOn: aStream [
	<category: 'printing'>
	super printOn: aStream.
	aStream nextPut: ${.
	self propDict keysAndValuesDo: 
		[:k :v | 
		aStream
		    print: k;
		    nextPutAll: '->';
		    print: v;
		    space].
	aStream nextPut: $}
    ]

    propDict [
	<category: 'accessing'>
	propDict isNil ifTrue: [propDict := IdentityDictionary new].
	^propDict
    ]

    propDict: aValue [
	<category: 'accessing'>
	propDict := aValue
    ]

    soapAction [
	<category: 'actions'>
	^self at: #soapAction ifAbsent: [SoapAction intentByRequestURI]
    ]

    soapAction: aStringOrSoapAction [
	<category: 'actions'>
	| soapAction |
	soapAction := aStringOrSoapAction isString 
		    ifTrue: [SoapAction intent: aStringOrSoapAction]
		    ifFalse: [aStringOrSoapAction].
	self at: #soapAction put: soapAction
    ]

    targetObjectURI [
	<category: 'actions'>
	^self at: #targetObjectURI ifAbsent: ['']
    ]

    targetObjectURI: aString [
	<category: 'actions'>
	self at: #targetObjectURI put: (SoPortableUtil relativeURIFrom: aString)
    ]
]



Object subclass: SoapDataBinder [
    
    <category: 'SOAP-Core-DataBinding'>
    <comment: nil>

    SoapDataBinder class [
	| typeDict |
	
    ]

    SoapDataBinder class >> initTypeDict [
	<category: 'class initialization'>
	^typeDict := IdentityDictionary new
    ]

    SoapDataBinder class >> initialize [
	<category: 'class initialization'>
	self initTypeDict
    ]

    SoapDataBinder class >> typeDict [
	<category: 'accessing'>
	typeDict isNil ifTrue: [self initTypeDict].
	^typeDict
    ]

    typeDict [
	<category: 'accessing'>
	^self class typeDict
    ]
]



SoapDataBinder subclass: SoapAbstractDecoder [
    | referenceDict |
    
    <category: 'SOAP-Core-Decoding'>
    <comment: nil>

    SoapAbstractDecoder class >> initTypeDict [
	"Apparently this mapping is not so sophisticted."

	"##TODO: Implement more serious mapping method"

	<category: 'class initialization'>
	| pref |
	typeDict := super initTypeDict.
	pref := SoapConstants xsdPrefixColon.
	#(#(#any #String) #(#anyType #String) #(#time #String) #(#boolean #Boolean) #(#float #Float) #(#double #Double) #(#integer #Integer) #(#positiveInteger #LargePositiveInteger) #(#negativeInteger #LargeNegativeInteger) #(#int #Integer) #(#string #String)) 
	    do: [:each | typeDict at: (pref , each first) asSymbol put: each last].
	^typeDict
    ]

    decodeXmlElement: aSoapWrapElement [
	<category: 'actions'>
	self subclassResponsibility
    ]

    initReferenceDict [
	<category: 'initialize-release'>
	referenceDict := nil
    ]

    referenceDict [
	"subclass may use this for resolving shared values"

	<category: 'accessing'>
	referenceDict ifNil: [referenceDict := Dictionary new].
	^referenceDict
    ]

    referenceDict: aDictionary [
	"subclass may use this for resolving shared values"

	<category: 'accessing'>
	referenceDict := aDictionary
    ]
]



SoapAbstractDecoder subclass: SoapNullDecoder [
    
    <category: 'SOAP-Core-Decoding'>
    <comment: nil>

    decodeXmlElement: aSoapWrapElement [
	<category: 'actions'>
	^aSoapWrapElement value
    ]
]



SoapAbstractDecoder subclass: SoapStandardDecoder [
    | complexTypeDict customComplexTypeDict arrayTypeDict |
    
    <category: 'SOAP-Core-Decoding'>
    <comment: nil>

    SoapStandardDecoder class [
	| complexTypeDict customComplexTypeDict arrayTypeDict |
	
    ]

    SoapStandardDecoder class >> arrayTypeDict [
	<category: 'accessing'>
	arrayTypeDict ifNil: [arrayTypeDict := Dictionary new].
	^arrayTypeDict
    ]

    SoapStandardDecoder class >> complexTypeDict [
	<category: 'accessing'>
	complexTypeDict ifNil: [complexTypeDict := Dictionary new].
	^complexTypeDict
    ]

    SoapStandardDecoder class >> customComplexTypeDict [
	<category: 'accessing'>
	customComplexTypeDict ifNil: [customComplexTypeDict := Dictionary new].
	^customComplexTypeDict
    ]

    SoapStandardDecoder class >> initTypeDict [
	"Apparently this mapping is not so sophisticted."

	"##TODO: Implement more serious mapping method"

	<category: 'class initialization'>
	| pref |
	typeDict := super initTypeDict.
	pref := SoapConstants xsdPrefixColon.
	#(#(#double #Float) #(#decimal #ScaledDecimal) #(#base64Binary #ByteArray) #(#long #LargePositiveInteger) #(#dateTime #DateTime)) 
	    do: [:each | typeDict at: (pref , each first) asSymbol put: each last].	"for Poor Squeak!!"
	^typeDict
    ]

    SoapStandardDecoder class >> initialize [
	"self initialize"

	<category: 'class initialization'>
	super initialize.
	complexTypeDict := Dictionary new.
	customComplexTypeDict := Dictionary new.
	arrayTypeDict := Dictionary new
    ]

    arrayTypeDict [
	"Answer the value of arrayTypeDict"

	<category: 'accessing'>
	arrayTypeDict ifNil: [arrayTypeDict := self class arrayTypeDict].
	^arrayTypeDict
    ]

    arrayTypeDict: anObject [
	"Set the value of customComplexTypeDict"

	<category: 'accessing'>
	arrayTypeDict := anObject
    ]

    basicDecodeXmlElement: aSoapWrapElement [
	<category: 'private'>
	| value |
	(self isArrayType: aSoapWrapElement) 
	    ifTrue: 
		[value := self decodeArrayXmlElement: aSoapWrapElement.
		^value].
	value := aSoapWrapElement children isEmpty 
		    ifTrue: 
			[self decodePrimitiveTyped: (self keyFor: aSoapWrapElement)
			    value: aSoapWrapElement value]
		    ifFalse: [self decodeComplexXmlElement: aSoapWrapElement].
	^value
    ]

    complexTypeDict [
	"Answer the value of complexTypeDict"

	<category: 'accessing'>
	complexTypeDict ifNil: [complexTypeDict := self class complexTypeDict].
	^complexTypeDict
    ]

    complexTypeDict: anObject [
	"Set the value of complexTypeDict"

	<category: 'accessing'>
	complexTypeDict := anObject
    ]

    customComplexTypeDict [
	"Answer the value of customComplexTypeDict"

	<category: 'accessing'>
	customComplexTypeDict 
	    ifNil: [customComplexTypeDict := self class customComplexTypeDict].
	^customComplexTypeDict
    ]

    customComplexTypeDict: anObject [
	"Set the value of customComplexTypeDict"

	<category: 'accessing'>
	customComplexTypeDict := anObject
    ]

    decodeArrayXmlElement: aSoapWrapElement [
	<category: 'private'>
	| sz arrayClass array |
	sz := aSoapWrapElement size.
	arrayClass := self arrayTypeDict at: (self keyFor: aSoapWrapElement)
		    ifAbsent: [Array].
	array := arrayClass new: sz.
	1 to: aSoapWrapElement children size
	    do: 
		[:idx | 
		| itemElem |
		itemElem := aSoapWrapElement children at: idx ifAbsent: [].
		itemElem isNil 
		    ifFalse: 
			[| itemType decodedItemValue |
			itemType := aSoapWrapElement arrayItemType.
			decodedItemValue := (((itemType = SoapConstants xsdAnyType 
				    or: [self isArrayType: itemElem]) or: [itemElem href notNil]) 
				    or: [itemElem children isEmpty not]) 
					ifTrue: [self basicDecodeXmlElement: itemElem]
					ifFalse: [self decodePrimitiveTyped: itemType value: itemElem value].
			array at: idx put: decodedItemValue]].
	^array
    ]

    decodeComplexXmlElement: aSoapXmlElement [
	<category: 'private'>
	| stClass |
	self customComplexTypeDict at: (self keyFor: aSoapXmlElement)
	    ifPresent: [:p | ^p fromXmlElement: aSoapXmlElement with: self].
	stClass := self complexTypeDict at: aSoapXmlElement xsiType
		    ifAbsent: 
			[self class environment at: (self keyFor: aSoapXmlElement)
			    ifAbsent: 
				[self class environment 
				    at: (SoXmlUtil localNameFrom: aSoapXmlElement xsiType)
				    ifAbsent: []]].
	stClass isNil 
	    ifTrue: 
		[| structDict |
		structDict := Dictionary new.
		aSoapXmlElement children do: 
			[:eachChild | 
			structDict at: (SoXmlUtil localNameFrom: eachChild name)
			    put: (self basicDecodeXmlElement: eachChild)].
		^structDict]
	    ifFalse: 
		[| structInst allInstVarNames |
		structInst := stClass new.
		allInstVarNames := stClass allInstVarNames.
		aSoapXmlElement children do: 
			[:eachChild | 
			structInst instVarAt: (allInstVarNames 
				    indexOf: (SoXmlUtil localNameFrom: eachChild name) 
					    withFirstCharacterDownshifted asSymbol)
			    put: (self basicDecodeXmlElement: eachChild)].
		^structInst]
    ]

    decodePrimitiveTyped: xsiTypeName value: value [
	"quick hack - should see both prefix url and type"

	<category: 'private'>
	| xsiTypeShortName stClsName stClass val |
	xsiTypeName isNil ifTrue: [^value].
	xsiTypeShortName := SoXmlUtil localNameFrom: xsiTypeName.
	xsiTypeShortName = SoapConstants boolean 
	    ifTrue: [^value = 'true' or: [value = '1']].
	xsiTypeShortName = SoapConstants string 
	    ifTrue: [^value isNil ifTrue: [''] ifFalse: [value]].
	xsiTypeShortName = SoapConstants time ifTrue: [^value].	"quick hack"
	(xsiTypeShortName = SoapConstants base64Binary 
	    or: [xsiTypeShortName = SoapConstants base64]) 
		ifTrue: [^(Base64MimeConverter mimeDecodeToBytes: value readStream) contents].
	stClsName := self typeDict at: xsiTypeName asSymbol
		    ifAbsent: 
			[SoapDecodingError 
			    signal: xsiTypeName , '''s decoding is not supported by ' 
				    , self class printString].
	stClass := self class environment at: stClsName.
	val := [stClass readFrom: (ReadStream on: value)] on: Error
		    do: 
			[:ex | 
			SoapDecodingError 
			    signal: ('Error in {1}>>readFrom: {2}' translated format: 
					{stClass.
					ex description}).
			nil].
	^val
    ]

    decodeXmlElement: aSoapWrapElement [
	<category: 'actions'>
	| value |
	value := self basicDecodeXmlElement: aSoapWrapElement.
	^value
    ]

    isArrayType: aSoapWrapElement [
	<category: 'private'>
	^aSoapWrapElement isArrayType
    ]

    keyFor: aSoapWrapElement [
	<category: 'private'>
	^aSoapWrapElement xsiType
    ]
]



SoapStandardDecoder subclass: SoapDecoder [
    
    <category: 'SOAP-Core-Decoding'>
    <comment: nil>

    SoapDecoder class [
	| useXsiNullAttribute |
	
    ]

    SoapDecoder class >> example1 [
	"SoapDecoder example1"

	<category: 'examples'>
	self customComplexTypeDict at: 'apachens:Map' put: SoapApachensMap
    ]

    SoapDecoder class >> initTypeDict [
	"Apparently this mapping is not so sophisticted."

	"##TODO: Implement more serious mapping method"

	<category: 'class initialization'>
	| pref |
	typeDict := super initTypeDict.
	pref := SoapConstants xsdPrefixColon.
	#(#(#double #Float) #(#decimal #ScaledDecimal) #(#base64Binary #ByteArray) #(#long #LargePositiveInteger) #(#dateTime #DateTime)) 
	    do: [:each | typeDict at: (pref , each first) asSymbol put: each last].	"for Poor Squeak!!"
	^typeDict
    ]

    SoapDecoder class >> initialize [
	"SoapDecoder initialize"

	<category: 'class initialization'>
	super initialize.
	useXsiNullAttribute := false
    ]

    SoapDecoder class >> useXsiNullAttribute [
	<category: 'setting'>
	^useXsiNullAttribute
    ]

    SoapDecoder class >> useXsiNullAttribute: aBoolean [
	<category: 'setting'>
	useXsiNullAttribute := aBoolean
    ]

    basicDecodeXmlElement: aSoapWrapElement [
	<category: 'private'>
	| value |
	aSoapWrapElement href ifNotNil: 
		[:href | 
		value := self referenceDict at: href ifAbsent: [].
		value ifNil: 
			[^self referenceDict at: href
			    put: (SoapHref on: href in: self referenceDict)].
		^value].
	self class useXsiNullAttribute 
	    ifTrue: 
		[value := self decodeNilXmlElement: aSoapWrapElement.
		value ifNil: [^value]].
	(self isArrayType: aSoapWrapElement) 
	    ifTrue: 
		[value := self decodeArrayXmlElement: aSoapWrapElement.
		self remember: value ifExistsId: aSoapWrapElement.
		^value].
	value := aSoapWrapElement children isEmpty 
		    ifTrue: 
			[self decodePrimitiveTyped: aSoapWrapElement xsiType
			    value: aSoapWrapElement value]
		    ifFalse: [self decodeComplexXmlElement: aSoapWrapElement].
	self remember: value ifExistsId: aSoapWrapElement.
	^value
    ]

    decodeNilXmlElement: aSoapWrapElement [
	<category: 'private'>
	| xsiNull |
	xsiNull := aSoapWrapElement getAttributeNamed: SoapConstants xsiNil.
	(xsiNull = '1' or: [xsiNull = 'true']) ifTrue: [^nil].
	^#NOT: NILL	"It is not nil"
    ]

    isArrayType: aSoapWrapElement [
	<category: 'private'>
	^(aSoapWrapElement isArrayType 
	    or: [self arrayTypeDict includesKey: aSoapWrapElement xsiType]) or: 
		    [self arrayTypeDict includesKey: aSoapWrapElement shortName	"temporal hack"]
    ]

    remember: object ifExistsId: aSoapXmlElement [
	<category: 'private'>
	aSoapXmlElement id 
	    ifNotNil: [:identifier | self referenceDict at: identifier put: object]
    ]
]



SoapDataBinder subclass: SoapAbstractEncoder [
    | referenceDict |
    
    <category: 'SOAP-Core-Encoding'>
    <comment: nil>

    SoapAbstractEncoder class >> initTypeDict [
	"Apparently this mapping is not so sophisticted."

	"##TODO: Implement more serious mapping method"

	<category: 'class initialization'>
	| pref |
	typeDict := super initTypeDict.
	pref := SoapConstants xsdPrefixColon.
	#(#(#Boolean #boolean) #(#Float #float) #(#FloatD #float) #(#FloatE #float) #(#Double #double) #(#Integer #integer) #(#SmallInteger #integer) #(#LargePositiveInteger #positiveInteger) #(#LargeNegativeInteger #negativeInteger) #(#String #string) #(#ByteString #string)) 
	    do: [:each | typeDict at: each first put: pref , each last].
	^typeDict
    ]

    encodeSoapVariable: aSoapVariable [
	<category: 'actions'>
	self subclassResponsibility
    ]

    isLiteralEncoder [
	<category: 'testing'>
	^true
    ]

    isSoapEncoder [
	<category: 'testing'>
	^false
    ]

    referenceDict [
	"subclass may use this for resolving shared values"

	<category: 'accessing'>
	referenceDict ifNil: [referenceDict := Dictionary new].
	^referenceDict
    ]

    referenceDict: aDictionary [
	"subclass may use this for resolving shared values"

	<category: 'accessing'>
	referenceDict := aDictionary
    ]

    reset [
	<category: 'initialize-release'>
	referenceDict := nil
    ]

    xmlElementClass [
	<category: 'factory'>
	^SoapWrapElement
    ]
]



SoapAbstractEncoder subclass: SoapNullEncoder [
    
    <category: 'SOAP-Core-Encoding'>
    <comment: nil>

    encodeSoapVariable: aSoapVariable [
	<category: 'actions'>
	| elem val |
	elem := self xmlElementClass named: aSoapVariable name.
	val := aSoapVariable value.
	elem putAttributeNamed: SoapConstants xsiType
	    value: SoapConstants nullEncPrefixColon , aSoapVariable value class name.
	elem basicValue: val.
	^elem
    ]
]



SoapAbstractEncoder subclass: SoapStandardEncoder [
    | complexTypeDict customComplexTypeDict |
    
    <category: 'SOAP-Core-Encoding'>
    <comment: nil>

    SoapStandardEncoder class [
	| complexTypeDict customComplexTypeDict |
	
    ]

    SoapStandardEncoder class >> complexTypeDict [
	<category: 'accessing'>
	complexTypeDict ifNil: [complexTypeDict := Dictionary new].
	^complexTypeDict
    ]

    SoapStandardEncoder class >> customComplexTypeDict [
	<category: 'accessing'>
	customComplexTypeDict ifNil: [customComplexTypeDict := Dictionary new].
	^customComplexTypeDict
    ]

    autoPrefixTo: anXmlElement [
	<category: 'private'>
	| autoPrefix |
	autoPrefix := SoXmlUtil randomPrefix.
	anXmlElement name: autoPrefix , ':' , anXmlElement.
	^autoPrefix
    ]

    complexTypeDict [
	"Answer the value of complexTypeDict"

	<category: 'accessing'>
	complexTypeDict ifNil: [complexTypeDict := self class complexTypeDict].
	^complexTypeDict
    ]

    complexTypeDict: anObject [
	"Set the value of complexTypeDict"

	<category: 'accessing'>
	complexTypeDict := anObject
    ]

    complexXsdTypeOf: anObject [
	<category: 'accessing'>
	^self complexTypeDict at: anObject class name ifAbsent: []
    ]

    customComplexTypeDict [
	"Answer the value of customComplexTypeDict"

	<category: 'accessing'>
	customComplexTypeDict 
	    ifNil: [customComplexTypeDict := self class customComplexTypeDict].
	^customComplexTypeDict
    ]

    customComplexTypeDict: anObject [
	"Set the value of customComplexTypeDict"

	<category: 'accessing'>
	customComplexTypeDict := anObject
    ]

    encodeArraySoapVariable: aSoapVariable [
	<category: 'private-encoding'>
	| arr elem allTypes elemType |
	arr := aSoapVariable value.
	arr class name == #SoapEncArray 
	    ifTrue: 
		[elem := arr trimForNested.	"temporary hack"
		self setXmlAttributesFrom: aSoapVariable into: elem.
		^elem].
	allTypes := arr collect: [:each | each class].
	elemType := (allTypes asSet size > 1 or: [allTypes isEmpty]) 
		    ifTrue: [SoapConstants xsdAnyType]
		    ifFalse: 
			[(self possibleTypeAt: allTypes first name) 
			    ifNil: [SoapConstants xsdAnyType]].
	elem := (SoapEncArray encoder: self) 
		    fromArray: arr
		    named: aSoapVariable name
		    elementType: elemType
		    containerType: aSoapVariable xsdType
		    namespace: aSoapVariable namespace.
	self setXmlAttributesFrom: aSoapVariable into: elem.
	^elem
    ]

    encodeDictionarySoapVariable: aSoapVariable [
	<category: 'private-encoding'>
	| elem dict |
	elem := self xmlElementClass named: aSoapVariable name.
	dict := aSoapVariable value.
	dict keysAndValuesDo: 
		[:k :v | 
		elem addChild: (self 
			    encodeSoapVariable: (SoapVariable name: k asString value: v))].
	self setXmlAttributesFrom: aSoapVariable into: elem.
	^elem
    ]

    encodeStructSoapVariable: aSoapVariable [
	<category: 'private-encoding'>
	| struct nm elem xsdType namespace |
	struct := aSoapVariable value.
	nm := aSoapVariable name isEmptyOrNil 
		    ifTrue: [struct name]
		    ifFalse: [aSoapVariable name].
	elem := self xmlElementClass named: nm.
	xsdType := aSoapVariable xsdType ifNil: [struct xsdType].
	xsdType ifNotNil: [elem xsiType: xsdType].
	namespace := aSoapVariable namespace ifNil: [struct namespace].
	namespace ifNotNil: 
		[| prefix |
		prefix := aSoapVariable guessedPrefix ifNil: [struct prefix].
		prefix ifNil: [prefix := self autoPrefixTo: elem].
		elem declareNamespace: prefix uri: namespace].
	struct keysAndValuesDo: 
		[:k :v | 
		elem addChild: (self 
			    encodeSoapVariable: (SoapVariable name: k asString value: v))].
	self setXmlAttributesFrom: aSoapVariable into: elem.
	^elem
    ]

    possibleTypeAt: typeName [
	<category: 'private'>
	^self typeDict at: typeName
	    ifAbsent: [self complexTypeDict at: typeName ifAbsent: []]
    ]

    setXmlAttributesFrom: aSoapVariable into: anXmlElement [
	<category: 'private'>
	self setXmlElementId: aSoapVariable into: anXmlElement.
	self setXmlElementHref: aSoapVariable into: anXmlElement.
	self setXmlElementNamespace: aSoapVariable into: anXmlElement.
	self setXmlElementPrefix: aSoapVariable into: anXmlElement
    ]

    setXmlElementHref: aSoapVariable into: anXmlElement [
	<category: 'private'>
	| href |
	href := aSoapVariable href.
	href isNil 
	    ifFalse: 
		[anXmlElement href: href.
		anXmlElement beEmpty].
	^anXmlElement
    ]

    setXmlElementId: aSoapVariable into: anXmlElement [
	<category: 'private'>
	| id |
	id := aSoapVariable id.
	id isNil ifFalse: [anXmlElement id: id].
	^anXmlElement
    ]

    setXmlElementNamespace: aSoapVariable into: anXmlElement [
	<category: 'private'>
	| ns |
	ns := aSoapVariable namespace.
	ns isNil 
	    ifFalse: 
		[| prefix |
		prefix := aSoapVariable guessedPrefix.
		prefix ifNil: [prefix := self autoPrefixTo: anXmlElement].
		anXmlElement declareNamespace: prefix uri: ns].
	^anXmlElement
    ]

    setXmlElementPrefix: aSoapVariable into: anXmlElement [
	<category: 'private'>
	| prefix |
	prefix := aSoapVariable prefix.
	prefix isEmptyOrNil 
	    ifFalse: 
		[aSoapVariable isPrefixedName 
		    ifFalse: [anXmlElement name: prefix , ':' , anXmlElement name]].
	^anXmlElement
    ]

    setXmlElementValue: aSoapVariable into: anXmlElement [
	"default, just set raw data"

	<category: 'private'>
	anXmlElement value: aSoapVariable value
    ]

    useSharedValuesWhile: block [
	<category: 'private'>
	block value
    ]
]



SoapStandardEncoder subclass: SoapEncoder [
    | useSharedValues |
    
    <category: 'SOAP-Core-Encoding'>
    <comment: 'I will do SOAP encoding which is documented in SOAP 1.1 (1.2) spec.
Note that I am only one of the eocodings for SOAP.

---
MU 10/5/2002 01:32'>

    SoapEncoder class [
	| useXsiNullAttribute useSharedValues |
	
    ]

    SoapEncoder class >> initTypeDict [
	"Apparently this mapping is not mature."

	"##TODO: Implement more serious mapping"

	<category: 'class initialiation'>
	| pref |
	typeDict := super initTypeDict.
	pref := SoapConstants xsdPrefixColon.
	#(#(#Any #any) #(#DateTime #dateTime) #(#Time #time) #(#Date #date) #(#Boolean #boolean) #(#True #boolean) #(#False #boolean) #(#Array #Array) #(#ByteArray #base64Binary) #(#Float #float) #(#Double #double) #(#Integer #integer) #(#SmallInteger #integer) #(#LargePositiveInteger #positiveInteger) #(#LargeNegativeInteger #negativeInteger) #(#Byte #byte) #(#String #string) #(#ByteString #string) #(#WideString #string) #(#Symbol #string) #(#ByteSymbol #string) #(#WideSymbol #string)) 
	    do: [:each | typeDict at: each first put: (pref , each last) asSymbol]
    ]

    SoapEncoder class >> initialize [
	"SoapEncoder initialize"

	<category: 'class initialiation'>
	super initialize.
	useXsiNullAttribute := false.
	useSharedValues := true
    ]

    SoapEncoder class >> useSharedValues [
	<category: 'setting'>
	^useSharedValues
    ]

    SoapEncoder class >> useSharedValues: aBoolean [
	<category: 'setting'>
	useSharedValues := aBoolean
    ]

    SoapEncoder class >> useXsiNullAttribute [
	<category: 'setting'>
	^useXsiNullAttribute
    ]

    SoapEncoder class >> useXsiNullAttribute: aBoolean [
	<category: 'setting'>
	useXsiNullAttribute := aBoolean
    ]

    arrayTypeCandidateNames [
	"Restricted to conservative"

	<category: 'defaults'>
	^#(#Array #OrderedCollection #SoapEncArray)
    ]

    convertedAsRef: aSoapVariable [
	<category: 'private'>
	| val pair masterXmlElem originalVar sharedId refVar |
	self useSharedValues ifFalse: [^nil].
	val := aSoapVariable value.
	(self shouldShareValue: val) ifFalse: [^nil].
	pair := self referenceDict at: (self keyFor: val) ifAbsent: [^nil].
	masterXmlElem := pair first.
	originalVar := pair second.
	sharedId := self keyFor: originalVar.
	refVar := SoapVariable name: aSoapVariable name hrefId: sharedId.
	masterXmlElem id: sharedId.
	^refVar
    ]

    dictionaryTypeCandidateNames [
	"Restricted to conservative"

	<category: 'defaults'>
	^#(#Dictionary #IdentityDictionary)
    ]

    encodeArraySoapVariable: aSoapVariable [
	<category: 'private-encoding'>
	| elem |
	elem := super encodeArraySoapVariable: aSoapVariable.
	self remember: elem for: aSoapVariable.
	^elem
    ]

    encodeDictionarySoapVariable: aSoapVariable [
	<category: 'private-encoding'>
	| elem |
	elem := super encodeDictionarySoapVariable: aSoapVariable.
	self remember: elem for: aSoapVariable.
	^elem
    ]

    encodeNilSoapVariable: aSoapVariable [
	<category: 'private-encoding'>
	| elem |
	^self class useXsiNullAttribute 
	    ifTrue: 
		[elem := self xmlElementClass emptyNamed: aSoapVariable name.
		elem putAttributeNamed: SoapConstants xsiNil value: 1.
		elem]
	    ifFalse: [nil]
    ]

    encodePrimitive: aSoapVariable typed: type [
	<category: 'private-encoding'>
	| elem xsiTypeAttVal |
	xsiTypeAttVal := aSoapVariable xsdType.
	xsiTypeAttVal isNil 
	    ifTrue: 
		[xsiTypeAttVal := self typeDict at: type
			    ifAbsent: 
				[SoapEncodingError 
				    signal: type printString , '''s encoding is not supported by ' 
					    , self class printString]].
	elem := self xmlElementClass named: aSoapVariable name.
	elem xsiType: xsiTypeAttVal asString.
	self setXmlAttributesFrom: aSoapVariable into: elem.
	self setXmlElementValue: aSoapVariable into: elem.
	self remember: elem for: aSoapVariable.
	^elem
    ]

    encodeRefSoapVariable: aSoapVariable [
	<category: 'private-encoding'>
	| elem |
	elem := self xmlElementClass named: aSoapVariable name.
	self setXmlAttributesFrom: aSoapVariable into: elem.
	^elem
    ]

    encodeSoapVariable: aSoapVariable [
	<category: 'actions'>
	| val type |
	aSoapVariable isRefValue 
	    ifTrue: [^self encodeRefSoapVariable: aSoapVariable].	"legacy support"
	(self convertedAsRef: aSoapVariable) 
	    ifNotNil: [:var | ^self encodeRefSoapVariable: var].
	val := aSoapVariable value.
	val isNil ifTrue: [^self encodeNilSoapVariable: aSoapVariable].
	type := aSoapVariable type.
	type isNil ifTrue: [type := val class name].
	self customComplexTypeDict at: type
	    ifPresent: [:p | ^aSoapVariable value asXmlElement: aSoapVariable with: self].
	(self arrayTypeCandidateNames includes: type) 
	    ifTrue: [^self encodeArraySoapVariable: aSoapVariable].
	(self dictionaryTypeCandidateNames includes: type) 
	    ifTrue: [^self encodeDictionarySoapVariable: aSoapVariable].
	type == #SoapEncStruct 
	    ifTrue: [^self encodeStructSoapVariable: aSoapVariable].
	self complexTypeDict at: type
	    ifPresent: 
		[:p | 
		^self encodeStructSoapVariable: (aSoapVariable 
			    value: ((SoapEncStruct from: aSoapVariable value) xsdType: p))].
	^self encodePrimitive: aSoapVariable typed: type
    ]

    encodeStructSoapVariable: aSoapVariable [
	<category: 'private-encoding'>
	| elem |
	elem := super encodeStructSoapVariable: aSoapVariable.
	self remember: elem for: aSoapVariable.
	^elem
    ]

    isSoapEncoder [
	<category: 'testing'>
	^true
    ]

    keyFor: anObject [
	<category: 'private'>
	^anObject soapIdKey
    ]

    remember: anXmlElement for: aSoapVariable [
	<category: 'private'>
	| val |
	self useSharedValues ifFalse: [^self].
	val := aSoapVariable value.
	(self shouldShareValue: val) 
	    ifTrue: 
		[self referenceDict at: (self keyFor: val)
		    put: 
			{anXmlElement.
			aSoapVariable}]
    ]

    setXmlElementValue: aSoapVariable into: anXmlElement [
	<category: 'private'>
	| type value |
	super setXmlElementValue: aSoapVariable into: anXmlElement.

	"otherwise"
	type := anXmlElement xsiType.
	value := anXmlElement value.
	type = SoapConstants xsdBase64BinaryType 
	    ifTrue: 
		[| encValue |
		encValue := (Base64MimeConverter mimeEncode: value asByteArray readStream) 
			    contents.
		^anXmlElement value: encValue]
    ]

    shouldShareValue: soapVarValue [
	"Currently, Boolean and Number types are not shared"

	<category: 'private'>
	soapVarValue == true ifTrue: [^false].
	soapVarValue == false ifTrue: [^false].
	soapVarValue isNumber ifTrue: [^false].
	^true
    ]

    useSharedValues [
	"Answer the value of useSharedValues"

	<category: 'accessing'>
	useSharedValues ifNil: [useSharedValues := self class useSharedValues].
	^useSharedValues
    ]

    useSharedValues: aBoolean [
	"Set the value of useSharedReferences"

	<category: 'accessing'>
	useSharedValues := aBoolean
    ]

    useSharedValuesWhile: block [
	<category: 'private'>
	| origUseSharedValues |
	origUseSharedValues := self useSharedValues.
	self useSharedValues: true.
	block value.
	self useSharedValues: origUseSharedValues
    ]
]



SoapStandardEncoder subclass: SoapLiteralEncoder [
    
    <category: 'SOAP-Core-Encoding'>
    <comment: 'I will do SOAP "literal'' encoding. 
---
MU 3/31/2007 16:17'>

    encodeSoapVariable: aSoapVariable [
	<category: 'actions'>
	| type elem |
	type := aSoapVariable value class name.
	(type == #Dictionary or: [type == #IdentityDictionary]) 
	    ifTrue: [^self encodeDictionarySoapVariable: aSoapVariable].
	type == #SoapEncStruct 
	    ifTrue: [^self encodeStructSoapVariable: aSoapVariable].
	self complexTypeDict at: type
	    ifPresent: 
		[:p | 
		^self encodeStructSoapVariable: (aSoapVariable
			    value: (SoapEncStruct from: aSoapVariable value);
			    yourself)].
	elem := self xmlElementClass named: aSoapVariable name.
	self setXmlAttributesFrom: aSoapVariable into: elem.
	self setXmlElementValue: aSoapVariable into: elem.
	^elem
    ]

    isLiteralEncoder [
	<category: 'testing'>
	^true
    ]
]



Object subclass: SoapDataBindingFactory [
    
    <category: 'SOAP-Core-DataBinding'>
    <comment: nil>

    SoapDataBindingFactory class [
	| dataBindingDict |
	
    ]

    DataBindingDict := nil.

    SoapDataBindingFactory class >> createFrom: anArrayOfEncodingStyleURI [
	<category: 'actions'>
	| uriArr key |
	anArrayOfEncodingStyleURI size > 1 
	    ifTrue: 
		[self noSuchDataBinder 
		    signal: 'Currently multi encodingStyleURI is not supported'].
	uriArr := (anArrayOfEncodingStyleURI isNil 
		    or: [anArrayOfEncodingStyleURI isEmpty]) 
			ifTrue: [self defaultEncodingStyleURIs]
			ifFalse: [anArrayOfEncodingStyleURI].
	key := uriArr first.	"##TODO: to support multi encoders"
	^(self dataBindingDict at: key
	    ifAbsent: [self noSuchDataBinder signal: key]) new
    ]

    SoapDataBindingFactory class >> dataBindingDict [
	<category: 'accessing'>
	dataBindingDict isNil 
	    ifTrue: 
		[dataBindingDict := Dictionary new.
		self initDataBindingDict].
	^dataBindingDict
    ]

    SoapDataBindingFactory class >> defaultEncodingStyleURIs [
	<category: 'default values'>
	^Array with: SoapConstants soapEncURI
    ]

    SoapDataBindingFactory class >> initDataBindingDict [
	"Default do nothing"

	<category: 'default values'>
	
    ]

    SoapDataBindingFactory class >> initialize [
	"SoapDataBindingFactory initialize"

	<category: 'class initialization'>
	dataBindingDict := nil
    ]

    SoapDataBindingFactory class >> initializeAll [
	"SoapDataBindingFactory initializeAll"

	<category: 'class initialization'>
	self allSubclassesDo: [:each | each initialize]
    ]

    SoapDataBindingFactory class >> noSuchDataBinder [
	<category: 'accessing-error'>
	^SoapNoSuchDataBinder
    ]

    SoapDataBindingFactory class >> uriEncoderDict [
	<category: 'accessing'>
	^UriEncoderDict
    ]
]



SoapDataBindingFactory subclass: SoapDecoderFactory [
    
    <category: 'SOAP-Core-Decoding'>
    <comment: nil>

    SoapDecoderFactory class >> initDataBindingDict [
	<category: 'default values'>
	self dataBindingDict at: SoapConstants nullEncURI put: SoapNullDecoder.
	self dataBindingDict at: SoapConstants soapEncURI put: SoapDecoder.
    ]

    SoapDecoderFactory class >> noSuchDataBinder [
	<category: 'accessing-error'>
	^SoapNoSuchDecoder
    ]
]



SoapDataBindingFactory subclass: SoapEncoderFactory [
    
    <category: 'SOAP-Core-Encoding'>
    <comment: nil>

    SoapEncoderFactory class >> initDataBindingDict [
	<category: 'default values'>
	self dataBindingDict at: SoapConstants nullEncURI put: SoapNullEncoder.
	self dataBindingDict at: SoapConstants soapEncURI put: SoapEncoder.
    ]

    SoapEncoderFactory class >> noSuchDataBinder [
	<category: 'accessing-error'>
	^SoapNoSuchEncoder
    ]
]



Object subclass: SoapDevUtil [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>

    SoapDevUtil class >> fileoutAllModules [
	"self fileoutAllModules"

	<category: 'file out'>
	#('SoXML-Base' 'SoXML-ParserAdapter' 'SOAP-Base' 'SOAP-Elements' 'SOAP-DataBinding' 'SOAP-Encoding' 'SOAP-Decoding' 'SOAP-ClientBase' 'SOAP-ServerBase' 'SOAP-Http' 'SOAP-Http-Client' 'SOAP-Http-Comanche' 'SOAP-RPC' 'SOAP-Example' 'SOAP-GoogleWebAPI' 'SOAP-TranslateExample' 'SOAP-CustomComplexType' 'SOAP-TestCases' 'SOAP-DataBinding-SRP' 'SOAP-TestCases-SRP' 'SOAP-DataBinding-SIXX' 'SOAP-TestCases-SIXX' 'SOAP-Sqtp' 'SOAP-Sqtp-Client' 'SOAP-Sqtp-Server' 'SOAP-TestCases-Sqtp') 
	    do: [:each | SystemOrganization fileOutCategory: each]
    ]

    SoapDevUtil class >> fileoutClientModules [
	"self fileoutClientModules"

	<category: 'file out'>
	#('SoXML-Base' 'SoXML-ParserAdapter' 'SOAP-Base' 'SOAP-Elements' 'SOAP-DataBinding' 'SOAP-Encoding' 'SOAP-Decoding' 'SOAP-ClientBase' 'SOAP-Http' 'SOAP-Http-Client' 'SOAP-RPC' 'SOAP-Example' 'SOAP-GoogleWebAPI' 'SOAP-TranslateExample' 'SOAP-CustomComplexType') 
	    do: [:each | SystemOrganization fileOutCategory: each]
    ]

    SoapDevUtil class >> fileoutServerModules [
	"self fileoutServerModules"

	<category: 'file out'>
	#('SoXML-Base' 'SoXML-ParserAdapter' 'SOAP-Base' 'SOAP-Elements' 'SOAP-DataBinding' 'SOAP-Encoding' 'SOAP-Decoding' 'SOAP-ServerBase' 'SOAP-Http' 'SOAP-Http-Comanche' 'SOAP-RPC' 'SOAP-Example') 
	    do: [:each | SystemOrganization fileOutCategory: each]
    ]
]



Object subclass: SoapElement [
    | attributesHolder subElements prefixName localName name value parent |
    
    <category: 'SOAP-Core-Elements'>
    <comment: 'An abstract class for SOAP element.


---
mu 5/21/2001 22:24'>

    SoapElement class >> addAttributesOf: anXmlWrapElement to: aSoapElement [
	<category: 'private'>
	anXmlWrapElement attributesDictionary 
	    keysAndValuesDo: [:k :v | aSoapElement putAttributeNamed: k value: v]
    ]

    SoapElement class >> addSubElementsOf: anXmlWrapElement to: aSoapElement [
	<category: 'private'>
	anXmlWrapElement children do: [:each | aSoapElement addSubElement: each]
    ]

    SoapElement class >> fromXmlElement: anXmlWrapElement [
	"Create a new SoapElement that wraps the passed XML element
	 or one of its children"

	<category: 'instance creation'>
	| inst |
	inst := self named: anXmlWrapElement name.
	self addAttributesOf: anXmlWrapElement to: inst.
	self addSubElementsOf: anXmlWrapElement to: inst.
	^inst
    ]

    SoapElement class >> localName [
	<category: 'constants'>
	^self name copyReplaceAll: 'Soap' with: ''
    ]

    SoapElement class >> named: anElementName [
	<category: 'instance creation'>
	| index readStream prefixName localName |
	index := anElementName indexOf: $:.
	index = 0 
	    ifTrue: [SoapNoNamespacePrefix signal: 'Namespace prefix must be provided!']
	    ifFalse: 
		[readStream := anElementName readStream.
		prefixName := readStream next: index - 1.
		readStream skip: 1.
		localName := readStream next: anElementName size - index].
	(localName isEmpty or: [localName ~= self localName]) 
	    ifTrue: 
		[SoapWrongTagName 
		    signal: '"' , localName , '"' , ' must be "' , self localName , '"'].
	^(self new)
	    prefixName: prefixName;
	    localName: localName;
	    name: anElementName
    ]

    SoapElement class >> new [
	<category: 'instance creation'>
	^super new initialize
    ]

    addChild: anXmlElement [
	<category: 'actions'>
	^self addSubElement: anXmlElement
    ]

    addSubElement: anXmlElement [
	"anXmlElement mustUnderstand:
	 #printXmlOn:aStream indent: anInteger
	 #parent: anXmlElement"

	<category: 'actions'>
	anXmlElement parent: self.
	^self subElements add: anXmlElement
    ]

    attributesHolder [
	<category: 'accessing'>
	^attributesHolder
    ]

    attributesHolder: aValue [
	<category: 'accessing'>
	attributesHolder := aValue
    ]

    declareDefaultNamespace [
	<category: 'actions'>
	self declareNamespace: SoapConstants soapEnvPrefix
	    uri: SoapConstants soapEnvURI
    ]

    declareEncodingIn: nameSpacePrefix uri: uriString [
	<category: 'actions'>
	self 
	    putAttributeNamed: nameSpacePrefix , ':' , SoapConstants encodingStyle
	    value: uriString
    ]

    declareNamespace: prefix uri: uri [
	<category: 'actions'>
	self attributesHolder declareNamespace: prefix uri: uri
    ]

    declareSoapEncEncoding [
	<category: 'actions'>
	self declareEncodingIn: SoapConstants soapEnvPrefix
	    uri: SoapConstants soapEncURI
    ]

    declareSoapEncNamespace [
	<category: 'actions'>
	self declareNamespace: SoapConstants soapEncPrefix
	    uri: SoapConstants soapEncURI
    ]

    declareTypicalNamespaces [
	<category: 'actions'>
	self declareXsdNamespace.
	self declareXsiNamespace
    ]

    declareXsdNamespace [
	<category: 'actions'>
	self declareNamespace: SoapConstants xsdPrefix uri: SoapConstants xsdURI
    ]

    declareXsiNamespace [
	<category: 'actions'>
	self declareNamespace: SoapConstants xsiPrefix uri: SoapConstants xsiURI
    ]

    declaredNamespacePrefixes [
	<category: 'actions'>
	^self attributesHolder declaredNamespacePrefixes
    ]

    encodingStyleURIs [
	"ToDo: take a more serious approach...Namespace conscious"

	<category: 'actions'>
	| encs |
	encs := self attributesHolder encodingStyle.
	self parent notNil 
	    ifTrue: 
		[| parentEncodeingURIs myURIs |
		parentEncodeingURIs := self parent encodingStyleURIs.
		myURIs := SoPortableUtil substringsFrom: encs.
		^(parentEncodeingURIs ~= myURIs and: [myURIs notEmpty]) 
		    ifTrue: [myURIs asArray]
		    ifFalse: [parentEncodeingURIs]].
	^(SoPortableUtil substringsFrom: encs) asArray
    ]

    evaluate [
	"default do nothing"

	<category: 'actions'>
	
    ]

    findSubElementLocalNamed: anElemName [
	<category: 'actions'>
	^self subElements detect: [:each | each localName = anElemName] ifNone: []
    ]

    findSubElementNamed: anElemName [
	<category: 'actions'>
	^self subElements detect: [:each | each name = anElemName] ifNone: []
    ]

    findSubElementsLocalNamed: anElemName [
	<category: 'actions'>
	^self subElements select: [:each | each localName = anElemName]
    ]

    findSubElementsNamed: anElemName [
	<category: 'actions'>
	^self subElements select: [:each | each name = anElemName]
    ]

    getAttributeNamed: aNameString [
	<category: 'actions'>
	^self attributesHolder getAttributeNamed: aNameString
    ]

    initialize [
	<category: 'initialize-release'>
	attributesHolder := SoapAttributesHolder new
    ]

    isBody [
	<category: 'testing'>
	^false
    ]

    isEnvelope [
	<category: 'testing'>
	^false
    ]

    isFault [
	<category: 'testing'>
	^false
    ]

    isFaultDetail [
	<category: 'testing'>
	^false
    ]

    isHeader [
	<category: 'testing'>
	^false
    ]

    localName [
	<category: 'accessing'>
	^localName
    ]

    localName: aString [
	<category: 'accessing'>
	name := nil.
	localName := aString
    ]

    name [
	<category: 'accessing'>
	name isNil 
	    ifTrue: 
		[prefixName := SoapConstants soapEnvPrefix.
		localName := self class localName.
		name := prefixName , ':' , localName].
	^name
    ]

    name: aString [
	<category: 'accessing'>
	name := aString
    ]

    parent [
	<category: 'accessing'>
	^parent
    ]

    parent: aSoapElement [
	<category: 'accessing'>
	parent := aSoapElement
    ]

    parser [
	<category: 'factory'>
	^self class parser
    ]

    prefixName [
	<category: 'accessing'>
	^prefixName
    ]

    prefixName: aString [
	<category: 'accessing'>
	name := nil.
	prefixName := aString
    ]

    printOn: aStream [
	"super printOn: aStream"

	"for Debug"

	<category: 'printing'>
	self printXmlOn: aStream
    ]

    printXmlContentsOn: aStream indent: level [
	<category: 'private-printing'>
	aStream cr.
	self privPrintChildrenOn: aStream indent: level + 1.
	level timesRepeat: [aStream space]
    ]

    printXmlFooterOn: aStream [
	<category: 'private-printing'>
	aStream nextPut: $<.
	aStream nextPut: $/.
	self writeName: self name on: aStream.
	aStream nextPut: $>
    ]

    printXmlHeaderOn: aStream [
	<category: 'private-printing'>
	aStream nextPut: $<.
	self writeName: self name on: aStream.
	self privPrintAttribsOn: aStream.
	aStream nextPut: $>
    ]

    printXmlOn: stream [
	<category: 'printing'>
	self printXmlOn: stream indent: 0
    ]

    printXmlOn: aStream indent: level [
	<category: 'private-printing'>
	level timesRepeat: [aStream space].
	self printXmlHeaderOn: aStream.
	self printXmlContentsOn: aStream indent: level.
	self printXmlFooterOn: aStream.
	aStream cr
    ]

    printXmlString [
	<category: 'printing'>
	| stream |
	stream := WriteStream on: (String new: 1024).
	self printOn: stream.
	^stream contents
    ]

    privPrintAttribsOn: aStream [
	<category: 'private-printing'>
	self attributesHolder printXmlOn: aStream
    ]

    privPrintChildrenOn: aStream indent: level [
	"$override$"

	<category: 'private-printing'>
	subElements isNil 
	    ifFalse: 
		[self subElements do: [:each | each printXmlOn: aStream indent: level]]
    ]

    putAttribute: aNameValueArray [
	<category: 'actions'>
	^self attributesHolder putAttribute: aNameValueArray
    ]

    putAttributeNamed: attribName value: attribValue [
	<category: 'actions'>
	^self attributesHolder putAttributeNamed: attribName value: attribValue
    ]

    subElements [
	<category: 'accessing'>
	subElements isNil ifTrue: [subElements := OrderedCollection new].
	^subElements
    ]

    subElements: aValue [
	<category: 'accessing'>
	subElements := aValue
    ]

    undeclareNamespaces [
	<category: 'actions'>
	self attributesHolder undeclareNamespaces
    ]

    value [
	<category: 'accessing'>
	^value
    ]

    value: aString [
	<category: 'accessing'>
	value := aString
    ]

    writeName: aCharacterArray on: aStream [
	"ToDo: check illegal characters"

	<category: 'private'>
	^aStream nextPutAll: aCharacterArray
    ]

    writeValue: aCharacterArray on: aStream [
	<category: 'private'>
	^SoXmlUtil writeXmlText: aCharacterArray on: aStream
    ]

    writeValue: aCharacterArray on: aStream betweenTag: aTagString [
	"ToDo: check illegal characters"

	<category: 'private'>
	aStream nextPutAll: '<' , aTagString , '>'.
	SoXmlUtil writeXmlText: aCharacterArray on: aStream.
	aStream nextPutAll: '</' , aTagString , '>'
    ]
]



SoapElement subclass: SoapBody [
    | fault bodyEntries |
    
    <category: 'SOAP-Core-Elements'>
    <comment: 'SOAP Body element.


---
mu 5/21/2001 22:24'>

    SoapBody class >> fromXmlElement: anXmlWrapElement [
	<category: 'instance creation'>
	| inst faultElem |
	inst := self named: anXmlWrapElement name.
	self addAttributesOf: anXmlWrapElement to: inst.
	faultElem := anXmlWrapElement findChildLocalNamed: SoapFault localName.
	faultElem isNil 
	    ifTrue: [anXmlWrapElement children do: [:each | inst addBodyEntry: each]]
	    ifFalse: [inst fault: (SoapFault fromXmlElement: faultElem)].
	^inst
    ]

    SoapBody class >> localName [
	<category: 'constants'>
	^'Body'
    ]

    addBodyEntry: anXmlElement [
	"anXmlElement mustUnderstand:
	 #printXmlOn:aStream
	 #parent: anXmlElement"

	<category: 'actions'>
	anXmlElement parent: self.
	^self bodyEntries add: anXmlElement
    ]

    addChild: anXmlElement [
	<category: 'actions'>
	^self addBodyEntry: anXmlElement
    ]

    addFault [
	<category: 'actions'>
	| fa |
	fa := SoapFault new.
	fa parent: self.
	self fault: fa.
	^fa
    ]

    bodyEntries [
	<category: 'elements'>
	bodyEntries isNil ifTrue: [bodyEntries := OrderedCollection new].
	^bodyEntries
    ]

    bodyEntries: aCollectionOfXmlElement [
	<category: 'elements'>
	bodyEntries := aCollectionOfXmlElement
    ]

    clearBodyEntries [
	<category: 'actions'>
	self bodyEntries do: [:each | each parent: nil].
	self bodyEntries: nil
    ]

    fault [
	<category: 'elements'>
	^fault
    ]

    fault: aSoapFault [
	<category: 'elements'>
	fault := aSoapFault.
	aSoapFault parent: self
    ]

    findBodyEntriesNamed: anElemName [
	<category: 'actions'>
	^self bodyEntries select: [:each | each name = anElemName]
    ]

    findBodyEntryNamed: anElemName [
	<category: 'actions'>
	^self bodyEntries detect: [:each | each name = anElemName] ifNone: []
    ]

    hasFault [
	<category: 'testing'>
	^self fault notNil
    ]

    isBody [
	<category: 'testing'>
	^false
    ]

    isMustUnderstandEnabled [
	<category: 'actions'>
	self parent isNil ifTrue: [^true].
	^self parent hasHeader 
	    ifTrue: [self parent header isMustUnderstandEnabled]
	    ifFalse: [true]
    ]

    privPrintChildrenOn: aStream indent: level [
	<category: 'private-printing'>
	fault isNil ifFalse: [self fault printXmlOn: aStream indent: level].
	bodyEntries isNil 
	    ifFalse: 
		[self bodyEntries do: [:each | each printXmlOn: aStream indent: level]].
	super privPrintChildrenOn: aStream indent: level
    ]
]



SoapElement subclass: SoapEnvelope [
    | header body |
    
    <category: 'SOAP-Core-Elements'>
    <comment: 'SOAP Envelope element.


---
mu 5/21/2001 22:24'>

    SoapEnvelope class >> fromXmlElement: anXmlWrapElement [
	<category: 'instance creation'>
	| inst headElem bodyElem others |
	inst := self named: anXmlWrapElement name.
	self addAttributesOf: anXmlWrapElement to: inst.
	headElem := anXmlWrapElement findChildLocalNamed: SoapHeader localName.
	headElem isNil 
	    ifFalse: [inst header: (SoapHeader fromXmlElement: headElem)].
	bodyElem := anXmlWrapElement findChildLocalNamed: SoapBody localName.
	bodyElem isNil ifFalse: [inst body: (SoapBody fromXmlElement: bodyElem)].
	others := anXmlWrapElement children reject: 
			[:each | 
			each localName = SoapHeader localName 
			    or: [each localName = SoapBody localName]].	"will be obsolete in SOAP 1.2"
	others do: [:otherE | inst addSubElement: otherE].
	^inst
    ]

    SoapEnvelope class >> localName [
	<category: 'constants'>
	^'Envelope'
    ]

    addBody [
	<category: 'actions'>
	| bo |
	bo := SoapBody new.
	bo parent: self.
	self body: bo.
	^bo
    ]

    addHeader [
	<category: 'actions'>
	| he |
	he := SoapHeader new.
	he parent: self.
	self header: he.
	^he
    ]

    body [
	<category: 'elements'>
	body isNil ifTrue: [self addBody].
	^body
    ]

    body: aSoapBody [
	<category: 'elements'>
	body := aSoapBody.
	aSoapBody parent: self
    ]

    hasFault [
	<category: 'testing'>
	^self body hasFault
    ]

    hasHeader [
	<category: 'testing'>
	^self header notNil
    ]

    header [
	<category: 'elements'>
	^header
    ]

    header: aSoapHeader [
	<category: 'elements'>
	header := aSoapHeader.
	aSoapHeader parent: self
    ]

    initialize [
	<category: 'initialize-release'>
	super initialize.
	self declareDefaultNamespace
    ]

    isEnvelope [
	<category: 'testing'>
	^true
    ]

    privPrintChildrenOn: aStream indent: level [
	<category: 'private-printing'>
	self header isNil ifFalse: [self header printXmlOn: aStream indent: level].
	self body printXmlOn: aStream indent: level.	"##body always exists##"
	super privPrintChildrenOn: aStream indent: level
    ]
]



SoapElement subclass: SoapFault [
    | faultcode faultstring faultactor detail |
    
    <category: 'SOAP-Core-Elements'>
    <comment: 'SOAP Fault element.


---
mu 5/21/2001 22:24'>

    SoapFault class >> client [
	<category: 'instance creation'>
	^self new client
    ]

    SoapFault class >> fromXmlElement: anXmlWrapElement [
	<category: 'instance creation'>
	| inst faultcode faultstring faultactor detail others |
	inst := self named: anXmlWrapElement name.
	self addAttributesOf: anXmlWrapElement to: inst.
	faultcode := anXmlWrapElement findChildNamed: 'faultcode'.
	faultcode isNil ifFalse: [inst faultcode: faultcode value].
	faultstring := anXmlWrapElement findChildNamed: 'faultstring'.
	faultstring isNil ifFalse: [inst faultstring: faultstring value].
	faultactor := anXmlWrapElement findChildNamed: 'faultactor'.
	faultactor isNil ifFalse: [inst faultactor: faultactor value].
	detail := anXmlWrapElement findChildNamed: 'detail'.
	detail isNil 
	    ifFalse: [inst detail: (SoapFaultDetail fromXmlElement: detail)].
	others := anXmlWrapElement children reject: 
			[:each | 
			#('faultcode' 'faultstring' 'faultactor' 'detail') includes: each name].
	others do: [:otherE | inst addSubElement: otherE].
	^inst
    ]

    SoapFault class >> localName [
	<category: 'constants'>
	^'Fault'
    ]

    SoapFault class >> mustUnderstand [
	<category: 'instance creation'>
	^self new mustUnderstand
    ]

    SoapFault class >> server [
	<category: 'instance creation'>
	^self new server
    ]

    SoapFault class >> versionMismatch [
	<category: 'instance creation'>
	^self new versionMismatch
    ]

    addDetail [
	<category: 'actions'>
	| de |
	de := SoapFaultDetail new.
	de parent: self.
	self detail: de.
	^de
    ]

    attachMinorFaultCodes: aArrayOfString [
	<category: 'actions'>
	| origFaultCode wStr |
	origFaultCode := self faultcode.
	wStr := WriteStream with: origFaultCode.
	aArrayOfString do: 
		[:each | 
		wStr nextPut: $..
		wStr nextPutAll: each].
	^self faultcode: wStr contents
    ]

    client [
	<category: 'actions-faultcode'>
	self faultcode: SoapConstants soapEnvPrefixColon , SoapConstants client
    ]

    detail [
	<category: 'elements'>
	^detail
    ]

    detail: aValue [
	<category: 'elements'>
	detail := aValue
    ]

    faultactor [
	<category: 'elements'>
	^faultactor
    ]

    faultactor: aValue [
	<category: 'elements'>
	faultactor := aValue
    ]

    faultcode [
	<category: 'elements'>
	^faultcode
    ]

    faultcode: aValue [
	<category: 'elements'>
	faultcode := aValue
    ]

    faultstring [
	<category: 'elements'>
	^faultstring
    ]

    faultstring: aValue [
	<category: 'elements'>
	faultstring := aValue
    ]

    initialize [
	<category: 'initialize-release'>
	super initialize.
	faultcode := faultstring := ''
    ]

    isFault [
	<category: 'testing'>
	^true
    ]

    mustUnderstand [
	<category: 'actions-faultcode'>
	self 
	    faultcode: SoapConstants soapEnvPrefixColon , SoapConstants mustUnderstand
    ]

    privPrintChildrenOn: aStream indent: level [
	<category: 'private-printing'>
	self 
	    writeValue: self faultcode
	    on: aStream
	    betweenTag: 'faultcode'.
	self 
	    writeValue: self faultstring
	    on: aStream
	    betweenTag: 'faultstring'.
	self faultactor isNil 
	    ifFalse: 
		[self 
		    writeValue: self faultactor
		    on: aStream
		    betweenTag: 'faultactor'].
	self detail isNil ifFalse: [self detail printXmlOn: aStream].
	super privPrintChildrenOn: aStream indent: level
    ]

    server [
	<category: 'actions-faultcode'>
	self faultcode: SoapConstants soapEnvPrefixColon , SoapConstants server
    ]

    versionMismatch [
	<category: 'actions-faultcode'>
	self 
	    faultcode: SoapConstants soapEnvPrefixColon , SoapConstants versionMismatch
    ]
]



SoapElement subclass: SoapFaultDetail [
    
    <category: 'SOAP-Core-Elements'>
    <comment: 'SOAP Fault Detail element.


---
mu 5/21/2001 22:24'>

    SoapFaultDetail class >> localName [
	<category: 'constants'>
	^'detail'
    ]

    SoapFaultDetail class >> named: anElementName [
	"No namespace prefix check"

	<category: 'instance creation'>
	^self new name: anElementName
    ]

    initialize [
	<category: 'initialize-release'>
	super initialize.
	name := self class localName
    ]

    isFaultDetail [
	<category: 'testing'>
	^true
    ]
]



SoapElement subclass: SoapHeader [
    | headerEntries |
    
    <category: 'SOAP-Core-Elements'>
    <comment: 'SOAP Header element.


---
mu 5/21/2001 22:24'>

    SoapHeader class >> localName [
	<category: 'constants'>
	^'Header'
    ]

    addHeaderEntry: anXmlElement [
	"anXmlElement mustUnderstand:
	 #printXmlOn:aStream
	 #parent: anXmlElement"

	<category: 'actions'>
	anXmlElement parent: self.
	^self headerEntries add: anXmlElement
    ]

    disableMustUnderstand [
	<category: 'actions'>
	self putAttributeNamed: SoapConstants soapEnvPrefixColon 
		    , SoapConstants mustUnderstand
	    value: '0'
    ]

    enableMustUnderstand [
	<category: 'actions'>
	self putAttributeNamed: SoapConstants soapEnvPrefixColon 
		    , SoapConstants mustUnderstand
	    value: '1'
    ]

    headerEntries [
	<category: 'elements'>
	headerEntries isNil ifTrue: [headerEntries := OrderedCollection new].
	^headerEntries
    ]

    headerEntries: aCollectionOfXmlElement [
	<category: 'elements'>
	headerEntries := aCollectionOfXmlElement
    ]

    isHeader [
	<category: 'testing'>
	^true
    ]

    isMustUnderstandEnabled [
	<category: 'actions'>
	^(self getAttributeNamed: SoapConstants soapEnvPrefixColon 
		    , SoapConstants mustUnderstand) 
	    = '1'
    ]
]



Object subclass: SoapEncStruct [
    | name xsdType namespace keysAndValues dictionary keys referenceHash |
    
    <category: 'SOAP-Core-Encoding'>
    <comment: nil>

    SoapEncStruct class >> from: object [
	<category: 'instance creation'>
	^self from: object named: nil
    ]

    SoapEncStruct class >> from: object named: elemName [
	<category: 'instance-creation-basic'>
	^self 
	    from: object
	    named: elemName
	    xsdType: nil
    ]

    SoapEncStruct class >> from: object named: elemName xsdType: xsdType [
	<category: 'instance-creation-basic'>
	^self 
	    from: object
	    named: elemName
	    xsdType: xsdType
	    namespace: nil
    ]

    SoapEncStruct class >> from: object named: elemName xsdType: xsdType namespace: namespaceURI [
	<category: 'instance-creation-basic'>
	| varNames assocs inst |
	varNames := object class allInstVarNames 
		    copyWithoutAll: object soapIgnorableInstVarNames.
	assocs := varNames collect: [:nm | nm -> (object instVarNamed: nm)].
	inst := self 
		    fromAssociations: assocs
		    named: elemName
		    xsdType: xsdType
		    namespace: namespaceURI.
	inst referenceHash: object identityHash.
	^inst
    ]

    SoapEncStruct class >> fromAssociations: keysAndValues [
	<category: 'instance creation'>
	^self fromAssociations: keysAndValues named: nil
    ]

    SoapEncStruct class >> fromAssociations: keysAndValues named: elemName [
	<category: 'instance-creation-basic'>
	^self 
	    fromAssociations: keysAndValues
	    named: elemName
	    xsdType: nil
    ]

    SoapEncStruct class >> fromAssociations: keysAndValues named: elemName xsdType: xsdType [
	<category: 'instance-creation-basic'>
	^self 
	    fromAssociations: keysAndValues
	    named: elemName
	    xsdType: xsdType
	    namespace: nil
    ]

    SoapEncStruct class >> fromAssociations: keysAndValues named: elemName xsdType: xsdType namespace: namespaceURI [
	<category: 'instance-creation-basic'>
	| inst |
	inst := (self new)
		    name: elemName;
		    xsdType: xsdType;
		    namespace: namespaceURI;
		    yourself.
	keysAndValues do: [:assoc | inst at: assoc key put: assoc value].
	^inst
    ]

    SoapEncStruct class >> fromAssociations: keysAndValues xsdType: xsdType namespace: namespaceURI [
	"obsolete"

	<category: 'instance-creation-basic'>
	^self 
	    fromAssociations: keysAndValues
	    named: xsdType
	    xsdType: xsdType
	    namespace: namespaceURI
    ]

    at: key [
	<category: 'actions'>
	^self dictionary at: key
    ]

    at: key ifAbsent: aBlock [
	<category: 'actions'>
	^self dictionary at: key ifAbsent: aBlock
    ]

    at: key put: value [
	"key is limited to String"

	"Note that it remembers order of key->value registrations"

	<category: 'actions'>
	| stringKey |
	stringKey := key asString.
	(self keys includes: stringKey) ifFalse: [self keys add: stringKey].
	^self dictionary at: stringKey put: value
    ]

    dictionary [
	<category: 'accessing'>
	dictionary ifNil: [dictionary := Dictionary new].
	^dictionary
    ]

    dictionary: aDictionary [
	<category: 'accessing'>
	dictionary := aDictionary
    ]

    keys [
	<category: 'accessing'>
	keys ifNil: [keys := OrderedCollection new].
	^keys
    ]

    keysAndValues [
	<category: 'actions'>
	| ord |
	ord := OrderedCollection new.
	self keysAndValuesDo: [:k :v | ord add: k -> v].
	^ord
    ]

    keysAndValuesDo: aBlock [
	<category: 'actions'>
	self keys 
	    do: [:eachKey | aBlock value: eachKey value: (self dictionary at: eachKey ifAbsent: [])]
    ]

    name [
	<category: 'accessing'>
	^name
    ]

    name: aString [
	<category: 'accessing'>
	name := aString
    ]

    namespace [
	<category: 'accessing'>
	^namespace
    ]

    namespace: namespaceURI [
	<category: 'accessing'>
	namespace := namespaceURI
    ]

    prefix [
	<category: 'accessing'>
	^SoXmlUtil prefixFrom: self name
    ]

    referenceHash [
	<category: 'accessing'>
	referenceHash ifNil: [referenceHash := self identityHash].
	^referenceHash
    ]

    referenceHash: hash [
	<category: 'accessing'>
	referenceHash := hash
    ]

    soapIdKey [
	<category: 'accessing'>
	^self referenceHash
    ]

    values [
	<category: 'actions'>
	| values |
	values := OrderedCollection new.
	self keys do: 
		[:eachKey | 
		(self dictionary at: eachKey ifAbsent: []) 
		    ifNotNil: [:v | values add: v]].
	^values
    ]

    xsdType [
	<category: 'accessing'>
	^xsdType
    ]

    xsdType: aString [
	<category: 'accessing'>
	xsdType := aString
    ]
]



Object subclass: SoapEncodable [
    | encodingStyleURIs encoder parent |
    
    <category: 'SOAP-Core-RPC'>
    <comment: nil>

    encoder [
	<category: 'accessing'>
	encoder isNil 
	    ifTrue: 
		[encoder := parent isNil 
			    ifTrue: [SoapEncoderFactory createFrom: self encodingStyleURIs]
			    ifFalse: [parent encoder]].
	^encoder
    ]

    encoder: aValue [
	<category: 'accessing'>
	encoder := aValue
    ]

    encoding: aSymbol [
	<category: 'actions'>
	self encodingStyleURI: (SoapEncodingStyleURIMap at: aSymbol).
	self encoder: (SoapEncoderFactory createFrom: self encodingStyleURIs)
    ]

    encodingStyleURI: aURIString [
	<category: 'actions'>
	self encodingStyleURIs: (Array with: aURIString)
    ]

    encodingStyleURIString [
	<category: 'actions'>
	| uris wStr |
	uris := self encodingStyleURIs.
	wStr := WriteStream on: (String new: 64).
	uris size = 1 
	    ifTrue: [wStr nextPutAll: uris first]
	    ifFalse: 
		[self encodingStyleURIs do: 
			[:each | 
			wStr
			    nextPutAll: each;
			    nextPut: Character space]].
	^wStr contents
    ]

    encodingStyleURIString: anEncodingStyleURI [
	<category: 'actions'>
	self 
	    encodingStyleURIs: (SoPortableUtil substringsFrom: anEncodingStyleURI)
    ]

    encodingStyleURIs [
	<category: 'accessing'>
	encodingStyleURIs isNil 
	    ifTrue: [encodingStyleURIs := Array with: SoapConstants soapEncURI].
	^encodingStyleURIs asArray
    ]

    encodingStyleURIs: aValue [
	<category: 'accessing'>
	encodingStyleURIs := aValue
    ]

    parent [
	<category: 'accessing'>
	^parent
    ]

    parent: aValue [
	<category: 'accessing'>
	parent := aValue
    ]
]



SoapEncodable subclass: SoapPRCElement [
    | targetObjectURI methodName parameters optionSignature optionHeaderProperties transport envelope context namespace prefix useLiteral style |
    
    <category: 'SOAP-Core-RPC'>
    <comment: nil>

    DefaultNamespace := nil.
    DefaultPrefix := nil.

    SoapPRCElement class >> defaultNamespace [
	<category: 'accessing'>
	DefaultNamespace isNil 
	    ifTrue: 
		[DefaultNamespace := 'http://www.mars.dti.ne.jp/~umejava/smalltalk/soapOpera/rpc/'].
	^DefaultNamespace
    ]

    SoapPRCElement class >> defaultNamespace: aString [
	<category: 'accessing'>
	DefaultNamespace := aString
    ]

    SoapPRCElement class >> defaultPrefix [
	<category: 'accessing'>
	DefaultPrefix isNil ifTrue: [DefaultPrefix := 'm'].
	^DefaultPrefix
    ]

    SoapPRCElement class >> defaultPrefix: aString [
	<category: 'accessing'>
	DefaultPrefix := aString
    ]

    SoapPRCElement class >> initialize [
	"SoapPRCElement initialize"

	<category: 'class initialization'>
	DefaultPrefix := DefaultNamespace := nil
    ]

    SoapPRCElement class >> unmarshallFrom: aSoapEnvelope [
	"create one of my family"

	<category: 'actions'>
	
    ]

    buildContext [
	<category: 'building context'>
	| ctx |
	ctx := SoapContext new.
	ctx targetObjectURI: self targetObjectURI.
	^ctx
    ]

    context [
	<category: 'accessing'>
	context isNil ifTrue: [context := self buildContext].
	^context
    ]

    context: aValue [
	<category: 'accessing'>
	context := aValue
    ]

    encoder: aValue [
	<category: 'actions-encoding'>
	parameters notNil 
	    ifTrue: [self parameters do: [:each | each encoder: aValue]].
	super encoder: aValue
    ]

    encoding: aSymbol [
	<category: 'actions-encoding'>
	parameters notNil 
	    ifTrue: [self parameters do: [:each | each encoding: aSymbol]].
	super encoding: aSymbol
    ]

    encodingStyleURI: aURIString [
	<category: 'actions-encoding'>
	parameters notNil 
	    ifTrue: [self parameters do: [:each | each encodingStyleURI: aURIString]].
	super encodingStyleURI: aURIString
    ]

    encodingStyleURIString: anEncodingStyleURI [
	<category: 'actions-encoding'>
	parameters notNil 
	    ifTrue: 
		[self parameters 
		    do: [:each | each encodingStyleURIString: anEncodingStyleURI]].
	super encodingStyleURIString: anEncodingStyleURI
    ]

    encodingStyleURIs: aValue [
	<category: 'actions-encoding'>
	parameters notNil 
	    ifTrue: [self parameters do: [:each | each encodingStyleURIs: aValue]].
	super encodingStyleURIs: aValue
    ]

    envelope [
	<category: 'accessing'>
	envelope isNil ifTrue: [envelope := SoapEnvelope new].
	^envelope
    ]

    envelope: aValue [
	<category: 'accessing'>
	envelope := aValue
    ]

    isCall [
	<category: 'testing'>
	^false
    ]

    isResponse [
	<category: 'testing'>
	^false
    ]

    isVariable [
	<category: 'testing'>
	^false
    ]

    methodName [
	<category: 'accessing'>
	methodName isNil ifTrue: [methodName := ''].
	^methodName
    ]

    methodName: aValue [
	<category: 'accessing'>
	methodName := aValue
    ]

    namespace [
	<category: 'accessing'>
	namespace isNil ifTrue: [namespace := self class defaultNamespace].
	^namespace
    ]

    namespace: aValue [
	<category: 'accessing'>
	namespace := aValue
    ]

    optionHeaderProperties [
	<category: 'accessing'>
	optionHeaderProperties isNil 
	    ifTrue: [optionHeaderProperties := Dictionary new].
	^optionHeaderProperties
    ]

    optionHeaderProperties: aValue [
	<category: 'accessing'>
	optionHeaderProperties := aValue
    ]

    optionSignature [
	<category: 'accessing'>
	^optionSignature
    ]

    optionSignature: aValue [
	<category: 'accessing'>
	optionSignature := aValue
    ]

    parameters [
	<category: 'accessing'>
	parameters isNil ifTrue: [parameters := OrderedCollection new].
	^parameters
    ]

    parameters: aValue [
	<category: 'accessing'>
	parameters := aValue
    ]

    prefix [
	<category: 'accessing'>
	prefix isNil ifTrue: [prefix := self class defaultPrefix].
	^prefix
    ]

    prefix: aValue [
	<category: 'accessing'>
	prefix := aValue
    ]

    style [
	<category: 'accessing'>
	style ifNil: [style := #rpc].
	^style
    ]

    style: aSymbol [
	"#rpc #document (note that #document is not 'document wrapped style')"

	<category: 'accessing'>
	| symbol |
	symbol := aSymbol asString asLowercase asSymbol.
	(#(#rpc #document) includes: symbol) ifFalse: [symbol := #rpc].
	^style := symbol
    ]

    targetObjectURI [
	<category: 'accessing'>
	targetObjectURI isNil ifTrue: [targetObjectURI := ''].
	^targetObjectURI
    ]

    targetObjectURI: aString [
	<category: 'accessing'>
	targetObjectURI := SoPortableUtil relativeURIFrom: aString
    ]

    transport [
	<category: 'accessing'>
	transport isNil ifTrue: [transport := #http].
	^transport
    ]

    transport: aValue [
	<category: 'accessing'>
	transport := aValue
    ]

    useLiteral [
	<category: 'accessing'>
	useLiteral isNil ifTrue: [useLiteral := false].
	^useLiteral
    ]

    useLiteral: aBoolean [
	<category: 'accessing'>
	useLiteral := aBoolean.
	useLiteral 
	    ifTrue: [self encoder: SoapLiteralEncoder new]
	    ifFalse: [self encoder isLiteralEncoder ifFalse: [self encoder: nil]]
    ]

    xmlElementClass [
	<category: 'factory'>
	^SoapWrapElement
    ]
]



SoapPRCElement subclass: SoapCall [
    | connector locator useColocation autoSoapActionIntent |
    
    <category: 'SOAP-Core-RPC'>
    <comment: 'I represent a Remote Procedure Call in SOAP.

---
MU 5/12/2002 00:47'>

    SoapCall class >> locator: aLocatorObject [
	<category: 'instance creation'>
	^self basicNew locator: aLocatorObject
    ]

    SoapCall class >> new [
	"default locator is SoapTcpLocator for localhost"

	<category: 'instance creation'>
	^self locator: (SoapTcpLocator host: 'localhost' port: 8823)
    ]

    SoapCall class >> url: anURLString [
	<category: 'instance creation'>
	| url host port protocol targetObjectURI locator inst |
	url := anURLString asUrl.
	host := url authority.
	port := url port.
	protocol := url schemeName asSymbol.
	targetObjectURI := url fullPath.
	locator := SoapTcpLocator host: host port: port.
	inst := self locator: locator.
	inst targetObjectURI: targetObjectURI.
	inst transport: protocol.
	^inst
    ]

    addParameter: aSoapVariable [
	<category: 'actions'>
	aSoapVariable parent: self.
	^self parameters add: aSoapVariable
    ]

    addParameterArray: parameterArray [
	"parameterArray -> {name. value(optional). xsdType(optional) }"

	<category: 'actions'>
	| paramSize |
	paramSize := parameterArray size.
	paramSize = 1 ifTrue: [^self addParameterNamed: (parameterArray at: 1)].
	paramSize = 2 
	    ifTrue: 
		[^self addParameterNamed: (parameterArray at: 1)
		    value: (parameterArray at: 2)].
	paramSize = 3 
	    ifTrue: 
		[^self 
		    addParameterNamed: (parameterArray at: 1)
		    value: (parameterArray at: 2)
		    xsdType: (parameterArray at: 3)]
    ]

    addParameterNamed: paramName [
	<category: 'actions'>
	^self addParameter: (SoapVariable name: paramName)
    ]

    addParameterNamed: paramName type: xsdType [
	<category: 'actions'>
	^self addParameter: (SoapVariable name: paramName type: xsdType)
    ]

    addParameterNamed: paramName value: paramValue [
	<category: 'actions'>
	^self addParameter: (SoapVariable name: paramName value: paramValue)
    ]

    addParameterNamed: paramName value: paramValue type: xsdType [
	<category: 'actions'>
	^self addParameter: (SoapVariable 
		    name: paramName
		    value: paramValue
		    type: xsdType)
    ]

    addParameterNamed: paramName value: paramValue xsdType: xsdType [
	<category: 'actions'>
	^self addParameter: (SoapVariable 
		    name: paramName
		    value: paramValue
		    xsdType: xsdType)
    ]

    addParameterNamed: paramName value: paramValue xsdType: xsdType namespace: namespaceURI [
	<category: 'actions'>
	^self addParameter: (SoapVariable 
		    name: paramName
		    value: paramValue
		    xsdType: xsdType
		    namespace: namespaceURI)
    ]

    addParameterNamed: paramName xsdType: xsdType [
	<category: 'actions'>
	^self addParameter: (SoapVariable name: paramName xsdType: xsdType)
    ]

    addParameterNamed: paramName xsdType: xsdType namespace: namespaceURI [
	<category: 'actions'>
	^self addParameter: (SoapVariable 
		    name: paramName
		    xsdType: xsdType
		    namespace: namespaceURI)
    ]

    addParameters: parameterArrays [
	<category: 'actions'>
	^parameterArrays do: [:each | self addParameterArray: each]
    ]

    autoSoapActionIntent [
	<category: 'accessing'>
	autoSoapActionIntent isNil ifTrue: [autoSoapActionIntent := false].
	^autoSoapActionIntent
    ]

    autoSoapActionIntent: aBoolean [
	<category: 'accessing'>
	autoSoapActionIntent := aBoolean
    ]

    buildEnvelope [
	<category: 'building envelope'>
	| env |
	env := self envelope.
	env declareTypicalNamespaces.
	self encoder reset.
	self useLiteral 
	    ifFalse: 
		[env declareEncodingIn: SoapConstants soapEnvPrefix
		    uri: self encodingStyleURIString.
		self encoder isSoapEncoder ifTrue: [env declareSoapEncNamespace]].
	self optionHeaderProperties isEmpty 
	    ifFalse: [self buildSoapHeader: env addHeader].
	env addBody.
	self buildSoapBody: env body.
	^env
    ]

    buildSoapBody: aSoapBody [
	<category: 'building envelope'>
	| callElem |
	callElem := self callElementFor: aSoapBody.
	self parameters do: [:each | callElem addChild: each asXmlElement].
	^aSoapBody
    ]

    buildSoapHeader: aSoapHeader [
	"##TODO: support additional header properties (like transaction id)"

	<category: 'building envelope'>
	^aSoapHeader
    ]

    callElementFor: aSoapBody [
	<category: 'private'>
	| pref callElem |
	(self style == #document and: [self methodName isEmpty]) 
	    ifTrue: [^aSoapBody].
	pref := self prefix.
	callElem := self xmlElementClass named: self methodName withPrefix: pref.
	callElem declareNamespace: pref uri: self namespace.
	aSoapBody addBodyEntry: callElem.
	^callElem
    ]

    clear [
	<category: 'private'>
	self optionHeaderProperties: nil.
	self envelope: nil.
	self context: nil
    ]

    colocated [
	<category: 'testing'>
	^self locator colocated
    ]

    connector [
	<category: 'accessing'>
	connector isNil ifTrue: [connector := self createConnector].
	^connector
    ]

    connector: aValue [
	<category: 'accessing'>
	connector := aValue
    ]

    createConnector [
	<category: 'factory'>
	| trans |
	trans := self transport.
	(self useColocation and: [self locator colocatedFor: self transport]) 
	    ifTrue: [trans := #localRpc].
	^SoapConnectorFactory createFrom: trans
    ]

    declareEnvelopeNamespace: prefixString uri: uri [
	<category: 'actions'>
	self envelope declareNamespace: prefixString uri: uri
    ]

    generateSoapActionIntent [
	"Especially for .Net server - that requires this type of SOAPAction format"

	<category: 'private'>
	self soapAction: self namespace , '/' , self methodName
    ]

    invoke [
	<category: 'actions-invoking'>
	| returnedMsg resp |
	self autoSoapActionIntent ifTrue: [self generateSoapActionIntent].
	self optionHeaderProperties 
	    keysAndValuesDo: [:k :v | self context at: k put: v].
	returnedMsg := self connector 
		    send: self buildEnvelope
		    to: self locator
		    with: self context.
	resp := SoapResponse fromSoapMessage: returnedMsg.
	resp style: self style.
	self clear.
	^resp
    ]

    invokeAndReturn [
	<category: 'actions-invoking'>
	| resp |
	resp := self invoke.
	^resp isFault ifTrue: [resp raiseException] ifFalse: [resp returnValue]
    ]

    invokeAndReturnHandling: handlerBlock [
	<category: 'actions-invoking'>
	^[self invokeAndReturn] on: SoapError , SoapImplError
	    do: [:ex | handlerBlock value: ex]
    ]

    invokeAndReturnWithValue: value [
	<category: 'actions-invoking'>
	| resp |
	resp := self invokeWithValue: value.
	^resp isFault ifTrue: [resp raiseException] ifFalse: [resp returnValue]
    ]

    invokeAndReturnWithValues: values [
	<category: 'actions-invoking'>
	| resp |
	resp := self invokeWithValues: values.
	^resp isFault ifTrue: [resp raiseException] ifFalse: [resp returnValue]
    ]

    invokeWithHeaderProperties: aHeaderProperties [
	<category: 'actions-invoking'>
	self headerProperties: aHeaderProperties.
	^self invoke
    ]

    invokeWithValue: value [
	<category: 'actions-invoking'>
	^self invokeWithValues: (Array with: value)
    ]

    invokeWithValues: values [
	"assuming parameters had been added"

	<category: 'actions-invoking'>
	| paramSize |
	paramSize := self parameters size.
	1 to: paramSize
	    do: [:idx | (self parameters at: idx) value: (values at: idx ifAbsent: [])].
	^self invoke
    ]

    isCall [
	<category: 'testing'>
	^true
    ]

    locator [
	<category: 'accessing'>
	locator isNil ifTrue: [locator := SoapCallEntry local locator].
	^locator
    ]

    locator: aValue [
	<category: 'accessing'>
	locator := aValue
    ]

    soapAction [
	<category: 'accessing'>
	^self context soapAction
    ]

    soapAction: aStringOrSoapAction [
	<category: 'accessing'>
	self context soapAction: aStringOrSoapAction
    ]

    targetObjectURI: aString [
	<category: 'accessing'>
	super targetObjectURI: aString
    ]

    transport: aSymbol [
	"need to reset connector"

	<category: 'actions'>
	self connector: nil.
	super transport: aSymbol
    ]

    useColocation [
	<category: 'accessing'>
	useColocation isNil ifTrue: [useColocation := false].
	^useColocation
    ]

    useColocation: aBoolean [
	<category: 'accessing'>
	aBoolean ifTrue: [self connector: nil].
	useColocation := aBoolean
    ]
]



SoapPRCElement subclass: SoapResponse [
    | returnValue outparameters fault exceptionBuilder referenceDict |
    
    <category: 'SOAP-Core-RPC'>
    <comment: 'I represent a SOAP RPC response.

---
MU 5/12/2002 00:47'>

    SoapResponse class [
	| useDotNetStyleSharedValues |
	
    ]

    SoapResponse class >> fromSoapMessage: aSoapMessage [
	<category: 'instance creation'>
	^self new initialize: aSoapMessage
    ]

    SoapResponse class >> initialize [
	<category: 'class initialization'>
	super initialize.
	useDotNetStyleSharedValues := false
    ]

    SoapResponse class >> useDotNetStyleSharedValues [
	<category: 'accessing'>
	useDotNetStyleSharedValues ifNil: [useDotNetStyleSharedValues := true].
	^useDotNetStyleSharedValues
    ]

    SoapResponse class >> useDotNetStyleSharedValues: aBoolean [
	<category: 'accessing'>
	useDotNetStyleSharedValues := aBoolean == true
    ]

    buildAdditionalResults [
	<category: 'private'>
	| retValues sz |
	retValues := self envelope body bodyEntries.
	retValues ifEmpty: [^retValues].
	sz := retValues size.
	sz <= 1 ifTrue: [^#()].
	2 to: sz do: [:idx | self extractDecodedValueFrom: (retValues at: idx)]
    ]

    buildDocumentTypeReturnValueWith: methodElem [
	<category: 'private'>
	| decoder retValues |
	decoder := self decoderFrom: methodElem.
	retValues := methodElem children.
	(retValues size = 1 and: [(decoder isArrayType: methodElem) not]) 
	    ifTrue: [^self extractDecodedValueFrom: retValues first].
	^retValues collect: [:each | self extractDecodedValueFrom: each]
    ]

    buildException [
	<category: 'private'>
	^self exceptionBuilder buildExceptionFrom: self fault
    ]

    buildOutparameters [
	<category: 'private'>
	| retValues retStruct sz outparams |
	retValues := self envelope body bodyEntries.
	retValues ifEmpty: [^retValues].
	retStruct := retValues first.
	sz := retStruct size.
	sz <= 1 ifTrue: [^#()].
	outparams := OrderedCollection new.
	2 to: sz
	    do: 
		[:idx | 
		outparams 
		    add: (self extractDecodedValueFrom: (retStruct children at: idx))].
	^outparams
    ]

    buildReturnValue [
	<category: 'private'>
	| retValueElem methodElem |
	methodElem := self envelope body bodyEntries first.
	self style = #document 
	    ifTrue: [^self buildDocumentTypeReturnValueWith: methodElem]
	    ifFalse: 
		[retValueElem := methodElem 
			    findChildNamed: SoapConstants returnValueAccessor.
		retValueElem isNil 
		    ifTrue: 
			[retValueElem := methodElem children isEmpty 
				    ifFalse: [methodElem children first]]].
	^self extractDecodedValueFrom: retValueElem
    ]

    decoderFrom: aSoapWrapElement [
	<category: 'private'>
	^SoapDecoderFactory createFrom: aSoapWrapElement encodingStyleURIs
    ]

    exceptionBuilder [
	<category: 'accessing'>
	exceptionBuilder isNil 
	    ifTrue: [exceptionBuilder := SoapExceptionBuilder new].
	^exceptionBuilder
    ]

    exceptionBuilder: aSoapExceptionBuilder [
	<category: 'accessing'>
	exceptionBuilder := aSoapExceptionBuilder
    ]

    extractDecodedValueFrom: aSoapWrapElement [
	<category: 'private'>
	| decoder |
	aSoapWrapElement isNil ifTrue: [^nil].
	decoder := self decoderFrom: aSoapWrapElement.
	decoder referenceDict: self referenceDict.
	^decoder decodeXmlElement: aSoapWrapElement
    ]

    fault [
	<category: 'accessing'>
	fault isNil ifTrue: [fault := self envelope body fault].
	^fault
    ]

    initReferenceDict [
	<category: 'initialize-release'>
	referenceDict := nil
    ]

    initialize: aSoapMessage [
	<category: 'initialize-release'>
	self initReferenceDict.
	self envelope: aSoapMessage envelope.
	self context: aSoapMessage context
    ]

    isFault [
	<category: 'actions'>
	^self fault notNil
    ]

    isResponse [
	<category: 'testing'>
	^true
    ]

    methodName [
	<category: 'accessing'>
	(self isFault not and: [methodName isNil]) 
	    ifTrue: 
		[methodName := self envelope body bodyEntries first name 
			    copyReplaceAll: 'Responce'
			    with: ''].
	^methodName
    ]

    outparameters [
	<category: 'actions'>
	self isFault ifTrue: [^nil].
	outparameters isNil 
	    ifTrue: 
		[outparameters := self buildOutparameters.
		self class useDotNetStyleSharedValues 
		    ifTrue: [self buildAdditionalResults]].
	^outparameters
    ]

    printOn: aStream [
	<category: 'printing'>
	aStream nextPutAll: 'a ' , self class name.
	aStream nextPutAll: ' ['.
	aStream nextPutAll: ' isFault= ' , self isFault printString.
	aStream nextPutAll: ' methodName= ' , self methodName printString.
	aStream nextPutAll: ' ]'
    ]

    raiseException [
	"If you would like to details, do it: (self fault inspect)"

	<category: 'actions'>
	self isFault ifTrue: [self buildException signal]
    ]

    referenceDict [
	<category: 'accessing'>
	referenceDict ifNil: [referenceDict := Dictionary new].
	^referenceDict
    ]

    referenceDict: aDictionary [
	<category: 'accessing'>
	referenceDict := aDictionary
    ]

    returnValue [
	<category: 'actions'>
	self isFault ifTrue: [^nil].
	returnValue isNil 
	    ifTrue: 
		[returnValue := self buildReturnValue.
		self class useDotNetStyleSharedValues 
		    ifTrue: [self buildAdditionalResults]].
	^returnValue
    ]

    targetObjectURI [
	<category: 'accessing'>
	targetObjectURI isNil 
	    ifTrue: [targetObjectURI := self context targetObjectURI].
	^targetObjectURI
    ]

    transport [
	<category: 'accessing'>
	transport isNil 
	    ifTrue: [transport := self context at: #transport ifAbsent: []].
	^transport
    ]
]



SoapEncodable subclass: SoapVariable [
    | name type id href value xsdType prefix namespace |
    
    <category: 'SOAP-Core-RPC'>
    <comment: 'I represent a SOAP RPC variable. Used as parameter and return value.

---
MU 5/12/2002 00:47'>

    SoapVariable class >> name: aNameSymbol [
	<category: 'instance creation'>
	^self 
	    name: aNameSymbol
	    value: nil
	    type: nil
    ]

    SoapVariable class >> name: aNameSymbol href: identifiable [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    href: '#' , identifiable assuredId
    ]

    SoapVariable class >> name: aNameSymbol hrefId: id [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    href: '#' , id asString
    ]

    SoapVariable class >> name: aNameSymbol type: aSmalltalkTypeSymbol [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    type: aSmalltalkTypeSymbol
    ]

    SoapVariable class >> name: aNameSymbol value: anObject [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    value: anObject
    ]

    SoapVariable class >> name: aNameSymbol value: anObject namespace: namespaceURI [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    value: anObject;
	    namespace: namespaceURI
    ]

    SoapVariable class >> name: aNameSymbol value: anObject prefix: prefix namespace: namespaceURI [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    value: anObject;
	    prefix: prefix;
	    namespace: namespaceURI
    ]

    SoapVariable class >> name: aNameSymbol value: anObject type: aSmalltalkTypeSymbol [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    value: anObject;
	    type: aSmalltalkTypeSymbol
    ]

    SoapVariable class >> name: aNameSymbol value: anObject xsdType: anXsdTypeSymbol [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    value: anObject;
	    xsdType: anXsdTypeSymbol
    ]

    SoapVariable class >> name: aNameSymbol value: anObject xsdType: anXsdTypeSymbol namespace: namespaceURI [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    value: anObject;
	    xsdType: anXsdTypeSymbol;
	    namespace: namespaceURI
    ]

    SoapVariable class >> name: aNameSymbol xsdType: anXsdTypeSymbol [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    xsdType: anXsdTypeSymbol
    ]

    SoapVariable class >> name: aNameSymbol xsdType: anXsdTypeSymbol namespace: namespaceURI [
	<category: 'instance creation'>
	^(self new)
	    name: aNameSymbol;
	    xsdType: anXsdTypeSymbol;
	    namespace: namespaceURI
    ]

    asXmlElement [
	<category: 'converting'>
	| elem |
	elem := self encoder encodeSoapVariable: self.
	(self parent isNil or: [self parent encoder class ~= self encoder class]) 
	    ifTrue: 
		[elem declareEncodingIn: SoapConstants soapEnvPrefix
		    uri: self encodingStyleURIString].
	^elem
    ]

    assuredId [
	<category: 'actions'>
	id isNil ifTrue: [id := self generateId].
	^id
    ]

    generateId [
	<category: 'private'>
	id := (Random new next * 1000000) truncated printString.
	^id
    ]

    guessedPrefix [
	<category: 'actions'>
	| xp p |
	^(xp := self xsdPrefix) isEmptyOrNil 
	    ifTrue: 
		[p := self prefix.
		p ifEmpty: [^nil].
		p]
	    ifFalse: [xp]
    ]

    href [
	<category: 'accessing'>
	^href
    ]

    href: aValue [
	<category: 'accessing'>
	href := aValue
    ]

    id [
	<category: 'accessing'>
	^id
    ]

    id: aValue [
	<category: 'accessing'>
	id := aValue
    ]

    isPrefixedName [
	<category: 'testing'>
	^SoXmlUtil isPrefixedName: self name
    ]

    isRefValue [
	<category: 'testing'>
	^self href notNil
    ]

    name [
	<category: 'accessing'>
	^name
    ]

    name: aValue [
	<category: 'accessing'>
	name := aValue
    ]

    namespace [
	<category: 'accessing'>
	^namespace
    ]

    namespace: aValue [
	<category: 'accessing'>
	namespace := aValue
    ]

    prefix [
	<category: 'accessing'>
	prefix ifNil: [prefix := SoXmlUtil prefixFrom: self name].
	^prefix
    ]

    prefix: aString [
	<category: 'accessing'>
	prefix := aString
    ]

    type [
	<category: 'accessing'>
	^type
    ]

    type: aValue [
	<category: 'accessing'>
	type := aValue
    ]

    value [
	<category: 'accessing'>
	self isRefValue ifTrue: [^nil].
	^value
    ]

    value: aValue [
	<category: 'accessing'>
	value := aValue
    ]

    xsdPrefix [
	<category: 'accessing'>
	^SoXmlUtil prefixFrom: self xsdType
    ]

    xsdType [
	<category: 'accessing'>
	^xsdType
    ]

    xsdType: aValue [
	<category: 'accessing'>
	xsdType := aValue
    ]
]



Object subclass: SoapEncodingStyleURIMap [
    
    <category: 'SOAP-Core-DataBinding'>
    <comment: nil>

    SoapEncodingStyleURIMap class [
	| ariasURIDict |
	
    ]

    SoapEncodingStyleURIMap class >> ariasURIDict [
	<category: 'accessing'>
	^ariasURIDict
    ]

    SoapEncodingStyleURIMap class >> at: anAriasSymbol [
	<category: 'actions'>
	^self ariasURIDict at: anAriasSymbol
	    ifAbsent: [SoapNoSuchEncodingArias signal: anAriasSymbol]
    ]

    SoapEncodingStyleURIMap class >> initAriasURIDict [
	<category: 'default values'>
	self ariasURIDict at: #nullEncoding put: SoapConstants nullEncURI.
	self ariasURIDict at: #soapEncoding put: SoapConstants soapEncURI.
    ]

    SoapEncodingStyleURIMap class >> initialize [
	"SoapEncodingStyleURIMap initialize"

	<category: 'class initialization'>
	ariasURIDict := IdentityDictionary new.
	self initAriasURIDict
    ]
]



Object subclass: SoapEnvelopeBuilder [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>

    buildClientSoapFaultEnvelope: anError [
	<category: 'actions'>
	| env fault builder |
	env := SoapEnvelope new.
	fault := env addBody addFault client.
	builder := SoapFaultDetailBuilder new.
	builder buildSoapFaultDetailOn: fault from: anError.
	^env
    ]

    buildServerSoapFaultEnvelope: anError [
	<category: 'actions'>
	| env fault builder |
	env := SoapEnvelope new.
	fault := env addBody addFault server.
	builder := SoapFaultDetailBuilder new.
	builder buildSoapFaultDetailOn: fault from: anError.
	^env
    ]

    buildSoapEnvelopeFromXmlString: anXmlString [
	<category: 'actions'>
	
	[| elem envelope |
	elem := SoapWrapElement fromXml: anXmlString.
	envelope := SoapEnvelope fromXmlElement: elem.
	envelope declareTypicalNamespaces.
	^envelope] 
		on: Error
		do: [:ex | ^self buildServerSoapFaultEnvelope: ex]
    ]
]



Object subclass: SoapErrorLog [
    
    <category: 'SOAP-Core-Base'>
    <comment: 'Write original error to a log file. 

---
mu 1/4/2002 22:51'>

    SoapErrorLog class >> write: anError [
	<category: 'actions'>
	| ex file stack |
	ex := anError.
	file := FileStream fileNamed: 'soapError.log'.
	file isNil 
	    ifTrue: 
		[Transcript
		    cr;
		    show: 'File open failed in SoapErrorLog'.
		^self].
	file setToEnd.
	file
	    cr;
	    nextPutAll: '### Original Error is... ###'.
	file
	    cr;
	    nextPutAll: TimeStamp current printString.
	file
	    cr;
	    nextPutAll: '#Receiver: ' , ex receiver printString.
	file
	    cr;
	    nextPutAll: '#Description: ' , ex description.
	stack := SoPortableUtil stackTraceFrom: ex.
	file
	    cr;
	    nextPutAll: '#Stack:'.
	file
	    cr;
	    cr;
	    nextPutAll: stack.
	file close
    ]
]



Object subclass: SoapExceptionBuilder [
    
    <category: 'SOAP-Core-RPC'>
    <comment: nil>

    buildExceptionFrom: aSoapFault [
	<category: 'actions'>
	| exception |
	aSoapFault detail isNil 
	    ifTrue: [exception := SoapError fromSoapFault: aSoapFault]
	    ifFalse: 
		[| exceptionDetail errorClassTag exceptionClassName messageText |
		exceptionDetail := aSoapFault detail subElements 
			    ifNotEmptyDo: [:elem | elem first].
		errorClassTag := exceptionDetail 
			    ifNotNil: [:elem | elem findChildNamed: 'ErrorClass'].
		exceptionClassName := errorClassTag isNil 
			    ifTrue: ['Error']
			    ifFalse: [errorClassTag value].
		exception := (Smalltalk at: exceptionClassName asSymbol
			    ifAbsent: [SoapNoSuchExceptionClass signal: exceptionClassName]) new.
		messageText := exceptionDetail 
			    ifNotNil: [:elem | elem findChildNamed: 'messageText'].
		exception messageText: (messageText isNil 
			    ifTrue: [aSoapFault faultstring]
			    ifFalse: [messageText value])].
	exception tag: '<<signaled in server>>'.
	"Should I use tag?"
	^exception
    ]
]



Object subclass: SoapFaultDetailBuilder [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>

    DefaultNamespace := nil.
    DefaultPrefix := nil.

    SoapFaultDetailBuilder class >> defaultNamespace [
	<category: 'accessing'>
	DefaultNamespace isNil 
	    ifTrue: 
		[DefaultNamespace := 'http://www.mars.dti.ne.jp/~umejava/smalltalk/soapOpera/error/'].
	^DefaultNamespace
    ]

    SoapFaultDetailBuilder class >> defaultNamespace: aString [
	<category: 'accessing'>
	DefaultNamespace := aString
    ]

    SoapFaultDetailBuilder class >> defaultPrefix [
	<category: 'accessing'>
	DefaultPrefix isNil ifTrue: [DefaultPrefix := 'e'].
	^DefaultPrefix
    ]

    SoapFaultDetailBuilder class >> defaultPrefix: aString [
	<category: 'accessing'>
	DefaultPrefix := aString
    ]

    SoapFaultDetailBuilder class >> initialize [
	"SoapFaultDetailBuilder initialize"

	<category: 'class initialization'>
	DefaultPrefix := DefaultNamespace := nil
    ]

    buildSoapFaultDetailOn: aSoapFault from: anError [
	<category: 'actions'>
	| errorPrefix details stErrorClass isResumable messageText stack detail |
	aSoapFault faultstring: anError description.
	errorPrefix := self class defaultPrefix.
	details := SoapWrapElement named: 'details' withPrefix: errorPrefix.
	details declareNamespace: errorPrefix uri: self class defaultNamespace.
	stErrorClass := details createChildNamed: 'ErrorClass'.
	stErrorClass value: anError class name.
	isResumable := details createChildNamed: 'isResumable'.
	isResumable value: anError isResumable printString.
	anError messageText isNil 
	    ifFalse: 
		[messageText := details createChildNamed: 'messageText'.
		messageText value: anError messageText].
	stack := details createChildNamed: 'stack'.
	stack value: (SoPortableUtil stackTraceFrom: anError).
	detail := aSoapFault addDetail.
	detail addSubElement: details.
	^detail
    ]
]



Object subclass: SoapMessage [
    | context envelope |
    
    <category: 'SOAP-Core-RPC'>
    <comment: nil>

    SoapMessage class >> envelope: aSoapEnvelope [
	<category: 'actions'>
	^self envelope: aSoapEnvelope context: nil
    ]

    SoapMessage class >> envelope: aSoapEnvelope context: aSoapContext [
	<category: 'actions'>
	^(self new)
	    envelope: aSoapEnvelope;
	    context: aSoapContext
    ]

    context [
	<category: 'accessing'>
	context isNil ifTrue: [context := SoapContext new].
	^context
    ]

    context: aValue [
	<category: 'accessing'>
	context := aValue
    ]

    envelope [
	<category: 'accessing'>
	^envelope
    ]

    envelope: aValue [
	<category: 'accessing'>
	envelope := aValue
    ]
]



Object subclass: SoapSetting [
    
    <category: 'SOAP-Core-Base'>
    <comment: 'Facade for setting SOAP related parameters

---
mu 1/4/2002 22:51'>

    SoapSetting class [
	| defaultTransport |
	
    ]

    SoapSetting class >> connectionTimeout [
	<category: 'transport setting'>
	^SoapAbstractSocketClient connectionTimeout
    ]

    SoapSetting class >> connectionTimeout: timeout [
	"SoapSetting connectionTimeout: 20"

	<category: 'transport setting'>
	SoapAbstractSocketClient connectionTimeout: timeout
    ]

    SoapSetting class >> defaultPort [
	"SoapSetting defaultPort"

	<category: 'server setting'>
	^SoapServerSetup defaultPortAt: self defaultTransport
    ]

    SoapSetting class >> defaultPort: anInteger [
	<category: 'server setting'>
	SoapServerSetup defaultPortAt: self defaultTransport put: anInteger.
	"Should I restart here?"
	self restart
    ]

    SoapSetting class >> defaultPortAt: tranportName [
	"SoapSetting defaultPortAt: #http"

	<category: 'server setting'>
	^SoapServerSetup defaultPortAt: tranportName
    ]

    SoapSetting class >> defaultPortAt: tranportName put: portNumber [
	"SoapSetting defaultPortAt: #http put: 8824"

	<category: 'server setting'>
	^SoapServerSetup defaultPortAt: tranportName put: portNumber
    ]

    SoapSetting class >> defaultSendingPort [
	"SoapSetting defaultSendingPort"

	<category: 'client setting'>
	^SoapCallEntry defaultSendingPort
    ]

    SoapSetting class >> defaultSendingPort: portNumber [
	"SoapSetting defaultSendingPort: 8823"

	<category: 'client setting'>
	^SoapCallEntry defaultSendingPort: portNumber
    ]

    SoapSetting class >> defaultTransport [
	<category: 'transport setting'>
	defaultTransport isNil ifTrue: [defaultTransport := #http].
	^defaultTransport
    ]

    SoapSetting class >> defaultTransport: aSymbol [
	<category: 'transport setting'>
	defaultTransport := aSymbol
    ]

    SoapSetting class >> initialize [
	"SoapSetting initialize"

	<category: 'class initialization'>
	defaultTransport := #http
    ]

    SoapSetting class >> resolveTimeout [
	<category: 'transport setting'>
	^SoapAbstractSocketClient resolveTimeout
    ]

    SoapSetting class >> resolveTimeout: timeout [
	"SoapSetting resolveTimeout: 20"

	<category: 'transport setting'>
	SoapAbstractSocketClient resolveTimeout: timeout
    ]

    SoapSetting class >> restart [
	<category: 'server setting'>
	SoapServerSetup startAll
    ]

    SoapSetting class >> showClientLog: aBoolean [
	"SoapSetting showClientLog: true"

	"SoapSetting showClientLog: false"

	<category: 'log setting'>
	SoapAbstractConnector showLog: aBoolean
    ]

    SoapSetting class >> showClientLogHttpLevel: aBoolean [
	"SoapSetting showClientLogHttpLevel: true"

	"SoapSetting showClientLogHttpLevel: false"

	<category: 'log setting'>
	SoapHttpClient showLog: aBoolean
    ]

    SoapSetting class >> showServerLog: aBoolean [
	"SoapSetting showServerLog: true"

	"SoapSetting showServerLog: false"

	<category: 'log setting'>
	Smalltalk at: #SoapHttpModule ifPresent: [:cls | cls showLog: aBoolean].
	Smalltalk at: #SoapSqtpHandler ifPresent: [:cls | cls showLog: aBoolean]
    ]

    SoapSetting class >> showingClientLog [
	"SoapSetting showingClientLog"

	<category: 'log setting'>
	^SoapAbstractConnector showLog
    ]

    SoapSetting class >> showingClientLogHttpLevel [
	"SoapSetting showingClientLogHttpLevel"

	<category: 'log setting'>
	^SoapHttpClient showLog
    ]

    SoapSetting class >> showingServerLog [
	"SoapSetting showingServerLog"

	<category: 'log setting'>
	| results |
	results := OrderedCollection new.
	results 
	    add: (Smalltalk at: #SoapHttpModule ifPresent: [:cls | cls showLog]).
	results 
	    add: (Smalltalk at: #SoapSqtpHandler ifPresent: [:cls | cls showLog]).
	^results includesAnyOf: (Array with: true with: nil)
    ]

    SoapSetting class >> stringEncoding [
	<category: 'encoding setting'>
	^SoXmlUtil stringEncoding
    ]

    SoapSetting class >> stringEncoding: aString [
	<category: 'encoding setting'>
	SoXmlUtil stringEncoding: aString
    ]

    SoapSetting class >> useHostAddressCache [
	<category: 'transport setting'>
	^SoapAbstractSocketClient useHostAddressCache
    ]

    SoapSetting class >> useHostAddressCache: aBoolean [
	"SoapSetting useHostAddressCache: true"

	"SoapSetting useHostAddressCache: false"

	<category: 'transport setting'>
	SoapAbstractSocketClient useHostAddressCache: aBoolean
    ]

    SoapSetting class >> useSchemaDatatypes1999 [
	<category: 'encoding setting'>
	SoXmlConstants xsiURI: 'http://www.w3.org/1999/XMLSchema'.
	SoXmlConstants xsiURI: 'http://www.w3.org/1999/XMLSchema-instance'
    ]

    SoapSetting class >> useSchemaDatatypes2001 [
	<category: 'encoding setting'>
	SoXmlConstants xsiURI: 'http://www.w3.org/2001/XMLSchema'.
	SoXmlConstants xsiURI: 'http://www.w3.org/2001/XMLSchema-instance'
    ]
]



Error subclass: SoapError [
    | faultcode faultstring detail |
    
    <category: 'SOAP-Core-Base'>
    <comment: 'SOAP Abstract Error class
---
mu 5/21/2001 22:30'>

    SoapError class >> client [
	<category: 'defaults'>
	^self faultcode: #Client
    ]

    SoapError class >> faultcode: aFaultCodeSymbol [
	<category: 'instance creation'>
	^self new faultcode: aFaultCodeSymbol
    ]

    SoapError class >> fromSoapFault: aSoapFault [
	<category: 'instance creation'>
	| inst |
	inst := self faultcode: aSoapFault faultcode.
	inst faultstring: aSoapFault faultstring.
	inst detail: aSoapFault detail.
	^inst
    ]

    SoapError class >> mustUnderstand [
	<category: 'defaults'>
	^self faultcode: #MustUnderstand
    ]

    SoapError class >> server [
	<category: 'defaults'>
	^self faultcode: #Server
    ]

    SoapError class >> versionMismatch [
	<category: 'defaults'>
	^self faultcode: #VersionMismatch
    ]

    description [
	<category: 'exceptionDescription'>
	| desc |
	desc := super description.
	self faultcode isNil ifFalse: [desc := desc , ':' , self faultcode].
	self faultstring isNil ifFalse: [desc := desc , '#' , self faultstring].
	^desc
    ]

    detail [
	<category: 'accessing'>
	^detail
    ]

    detail: aValue [
	<category: 'accessing'>
	detail := aValue
    ]

    faultCode [
	<category: 'accessing'>
	^faultcode
    ]

    faultCode: aValue [
	<category: 'accessing'>
	faultcode := aValue
    ]

    faultcode [
	<category: 'accessing'>
	^faultcode
    ]

    faultcode: aValue [
	<category: 'accessing'>
	faultcode := aValue
    ]

    faultstring [
	<category: 'accessing'>
	^faultstring
    ]

    faultstring: aValue [
	<category: 'accessing'>
	faultstring := aValue
    ]

    isResumable [
	<category: 'exceptionDescription'>
	^true
    ]
]



Error subclass: SoapImplError [
    
    <category: 'SOAP-Core-Base'>
    <comment: 'Abstract class for Soap implementation error.

---
mu 5/21/2001 22:30'>

    isResumable [
	<category: 'exceptionDescription'>
	^true
    ]
]



SoapImplError subclass: SoapDatabindingError [
    
    <category: 'SOAP-Core-DataBinding'>
    <comment: nil>
]



SoapDatabindingError subclass: SoapDecodingError [
    
    <category: 'SOAP-Core-Decoding'>
    <comment: nil>
]



SoapDatabindingError subclass: SoapEncodingError [
    
    <category: 'SOAP-Core-Encoding'>
    <comment: nil>
]



SoapImplError subclass: SoapInvalidParamName [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



SoapImplError subclass: SoapInvalidParamNumber [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



SoapImplError subclass: SoapNoNamespacePrefix [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



SoapImplError subclass: SoapNoSuchClass [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



SoapNoSuchClass subclass: SoapNoSuchExceptionClass [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



SoapNoSuchClass subclass: SoapNoSuchTransport [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



SoapImplError subclass: SoapNoSuchDataBinder [
    
    <category: 'SOAP-Core-DataBinding'>
    <comment: nil>
]



SoapNoSuchDataBinder subclass: SoapNoSuchDecoder [
    
    <category: 'SOAP-Core-Decoding'>
    <comment: nil>
]



SoapNoSuchDataBinder subclass: SoapNoSuchEncoder [
    
    <category: 'SOAP-Core-Encoding'>
    <comment: nil>
]



SoapImplError subclass: SoapNoSuchEncodingArias [
    
    <category: 'SOAP-Core-DataBinding'>
    <comment: nil>
]



SoapImplError subclass: SoapNoSuchService [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



SoapImplError subclass: SoapNoSuchType [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



SoapImplError subclass: SoapServiceInvocationFailed [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



SoapImplError subclass: SoapWrongTagName [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>
]



Warning subclass: SoapTimeoutError [
    
    <category: 'SOAP-Core-Base'>
    <comment: nil>

    defaultAction [
	<category: 'exceptionDescription'>
	Transcript
	    show: self class name , ': data was late';
	    cr.
	self resume
    ]
]



Object subclass: SoapHref [
    | target referenceDict realObject |
    
    <comment: nil>
    <category: 'SOAP-Core-Decoding'>

    SoapHref class >> on: target in: referenceDict [
	<category: 'instance creation'>
	^(self new)
	    target: target;
	    referenceDict: referenceDict;
	    yourself
    ]

    = anObject [
	^ self realObject = anObject
    ]

    size [
	^ self realObject size
    ]

    basicInspect [
	<category: 'actions'>
	
	{self class.
	
	{#target -> target.
	#referenceDict -> referenceDict.
	#realObject -> realObject}} 
		inspect
    ]

    becomeReal [
	<category: 'actions'>
	realObject isNil ifFalse: [^self safeBecomeForward: realObject].
	^self
    ]

    doesNotUnderstand: aMessage [
	<category: 'system primitives'>
	| real |
	real := self realObject.
	(real isMemberOf: self class) 
	    ifFalse: [^real perform: aMessage selector withArguments: aMessage arguments]
    ]

    inspect [
	<category: 'actions'>
	self realObject inspect
    ]

    realObject [
	<category: 'accessing'>
	realObject 
	    ifNil: [realObject := self referenceDict at: self target ifAbsent: [^self]].
	^realObject
    ]

    referenceDict [
	"Answer the value of referenceDict"

	<category: 'accessing'>
	^referenceDict
    ]

    referenceDict: anObject [
	"Set the value of referenceDict"

	<category: 'accessing'>
	referenceDict := anObject
    ]

    resolve [
	<category: 'actions'>
	self realObject
    ]

    safeBecomeForward: anObject [
	<category: 'private'>
	anObject == true ifTrue: [^anObject].
	anObject == false ifTrue: [^anObject].
	anObject isNumber ifTrue: [^anObject].
	anObject class isBits ifTrue: [^anObject].
	anObject class isVariable ifTrue: [^anObject].
	self becomeForward: anObject.
	^self
    ]

    target [
	"Answer the value of target"

	<category: 'accessing'>
	^target
    ]

    target: anObject [
	"Set the value of target"

	<category: 'accessing'>
	| str |
	str := anObject asString.
	str ifEmpty: [^nil].
	str := str copyFrom: 2 to: str size.	"removing #"
	target := str
    ]

    yourself [
	<category: 'actions'>
	^self
	    resolve;
	    becomeReal
    ]
]



Eval [
    SoapCallEntry initialize.
    SoapConfigurationMap initialize.
    SoapDecoder initialize.
    SoapStandardDecoder initialize.
    SoapEncoder initialize.
    SoapDataBinder initialize.
    SoapDataBindingFactory initialize.
    SoapPRCElement initialize.
    SoapResponse initialize.
    SoapEncodingStyleURIMap initialize.
    SoapFaultDetailBuilder initialize.
    SoapSetting initialize
]

